import qs from 'querystring';
import {combineReducers} from 'redux';
import {reducer as formReducer} from 'redux-form';
import {connectRouter} from 'connected-react-router'
import * as actionCreatorsConsts from './../action_creators_consts.js';
import moment from "moment";
import * as consts from "../consts";
import _ from "lodash";
import Immutable from "seamless-immutable";
import numeral from "numeral";
import {fromNowNoFuture, stringDuration} from "./../utils/time";
import {costTime} from "./../components/tabs/cost/costUtils";
import * as licenseUtils from './../utils/license-utils';
import {ignoreActions} from "redux-ignore";
import * as listActionsUtils from "../utils/listActionsUtils";
import {formatActivityValues, sendEvents, statusConvert} from "../utils/helpers";
import {decodeAll, encodeAll} from "../utils/encoders";
import {getEndPointType, removeCredentialsByType} from "../utils/sync-utils"
import {
    SETTINGS_DEFAULTS,
    removeSettingsFromSessionStorage
} from '../utils/settings-utils';

import {
    addUnhandledMessages,
    buildStateFromAction,
    formatBytes,
    setActivityInProgress,
    updateCurrentStep,
    updateErrorsPanelStatus,
    updateKeyWithBuiltState,
    updateLastRequestsForRelationshipsData
} from "../utils/reducerUtils"

import * as actions from '../action_creators_consts';

//const pendingRequestUpdater = (state, action) => state.update("_pendingRequests", pending => pending.set(action.payload.requestId, fromJS({requestId: action.payload.requestId, type: action.type})));


const handleErrors = (action, state) => {
    if (!actionCreatorsConsts.routesContainer.routes) return state;

    if (action.status === actionCreatorsConsts.STATUS_ERROR && action.type !== actionCreatorsConsts.LOGIN_LOCAL && !action.selfErrorHandling) {
        const thePayload = action.payload;
        const errorTime = moment();
        let errorMessage = "Unknown error";
        let errorKey = action.type;
        if (!_.isUndefined(thePayload)) {
            const gotErrorResponse = !_.isEmpty(thePayload);
            const isNetworkError = gotErrorResponse && thePayload.message === "Network Error";
            const isClientError = !gotErrorResponse;
            errorKey = thePayload.status === 504 || thePayload.status === 502 || isNetworkError ? "NETWORK" : action.type;
            errorMessage = thePayload.status === 504 || thePayload.status === 502 || isNetworkError ? "Please check your network connectivity and refresh the browser." : !isClientError ? (thePayload.data.message || thePayload.data.reason || thePayload.data) : `Client Error: ${thePayload.message}`;
            if (!errorMessage) errorMessage = thePayload.statusText ? thePayload.statusText : "Unknown error";


            //ugly patch to fix the firefox bug SER-394 During refresh of browser a connectivity error pops up in the client without a need, on messages.cancel the firefox throws it's own weired network error, we don't want to show it
            if (isNetworkError && thePayload.fileName && thePayload.fileName.indexOf("bundle.js") !== -1) {
                return state;
            }

            try {
                if (isClientError && console.error) {
                    console.error(action.payload)
                }
            } catch (e) {
                if (console.log) console.log(`Suppressed error ${action.payload.message}`);
            }
        }
        const error = {
            id: errorKey,
            message: errorMessage,
            time: errorTime
        };

        return updateErrorsPanelStatus(state.updateIn(["_generalErrors", "_errorsMap"], _errorsMap => _errorsMap.set(errorKey, error)));
    }

    if (action.status && action.status !== actionCreatorsConsts.STATUS_ERROR) {
        const updateErrorsMap = (errorsMap) => {
            const updatedErrorsMap = errorsMap.without(action.type).without(actionCreatorsConsts.action_update_map[action.type]);
            if (action.status === actionCreatorsConsts.STATUS_SUCCESS) return updatedErrorsMap.without("NETWORK");
            else return updatedErrorsMap;
        };

        return updateErrorsPanelStatus(state.updateIn(["_generalErrors", "_errorsMap"], updateErrorsMap));
    }

    else return state;
};

export const defaultState = (_config, _routing) => Immutable({
    _config,
    _routing,
    _ui: {clearContentHeight: false},
    unhandledMessages: [],
    _pendingRequests: [],
    _timeline: {data: []},
    _generalErrors: {_errorsMap: {}, _errorsPanelOpen: false, _errorsPanelSize: {}}
});

export const syncReducer = (initialState = defaultState(), action = {}) => {
    const state = handleErrors(action, initialState);
    const stateWithClearContentHeightFalse = state.setIn(["_ui", "clearContentHeight"], false);
    const stateWithWizardInfo = fillWizardData(stateWithClearContentHeightFalse);
    const updatedState = handleAction(stateWithWizardInfo, action);
    return handleUnhandledMessages(updatedState);
};


const processRelationship = (relationship) => {
    const isRunning = relationship.activity.status === consts.RESPONSE_STATUS_RUNNING;
    const lastTransferred = relationship.activity.lastMessageTime ? fromNowNoFuture(relationship.activity.lastMessageTime) : "No previous copy";
    const uploadScanning = relationship.activity.scanning;
    const calculatedProgress = uploadScanning ?
        `(preparing ${numeral(uploadScanning.objectsCompleted + uploadScanning.filesCompleted + uploadScanning.objectsIgnored).format('0.[0]a')})` :
        (isRunning && !_.isNil(relationship.activity.progress) ? `(${relationship.activity.progress}%)` : "");
    const duration = relationship.activity.executionTime ? stringDuration(relationship.activity.executionTime) : "No previous copy";
    const statusConverted = relationship.activity.status === consts.RESPONSE_STATUS_DONE ?
        (relationship.activity.filesFailed === 0 && relationship.activity.dirsFailedToScan === 0 ? 'SUCCESS' : 'COMPLETE' ) : relationship.activity.status;
    const lastTrasnferText = (relationship.activity.status === consts.RESPONSE_STATUS_FAILED || relationship.activity.status === consts.RESPONSE_STATUS_INTERRUPTED) ? lastTransferred :
        (isRunning ? `${relationship.activity.type} | Duration: ${duration}` : `${relationship.activity.type} | Duration: ${duration} | ${lastTransferred}` );
    const tags = relationship.target[relationship.target.protocol].tags;
    const dataBrokers = relationship.group.dataBrokers.map((dataBroker) => processAgent(dataBroker, undefined, isRunning));
    //if all the agents are in failed/waiting status, we want to show unknown status for a running copy/sync, or the last sync/copy status for a non currently running relationship.
    const showUnknown = isRunning && _.every(dataBrokers, (dataBroker) => (dataBroker.isFailed || dataBroker.isWaiting));
    relationship.ui = Object.assign({}, formatActivityValues(relationship.activity), {
        showUnknown,
        lastTransferred,
        lastDuration: duration,
        lastMessageTime: relationship.activity.lastMessageTime ? relationship.activity.lastMessageTime : "-",
        statusText: showUnknown ? consts.UNKNOWN_SYNC_STATUS : statusConvert[statusConverted],
        progress: calculatedProgress,
        status: relationship.activity.status,
        lastTransferredHeaderText: lastTrasnferText,
        failureMessage: relationship.activity.failureMessage,
        autoSync: relationship.sync.autoSync,
        sync: relationship.sync,
        actualTarget: relationship.actualTarget,
        source: relationship.source,
        gracePeriod: _.get(relationship.settings, 'gracePeriod', SETTINGS_DEFAULTS.gracePeriod),
        deletionOn:  _.get(relationship.settings, 'deleteOnTarget', SETTINGS_DEFAULTS.deleteOnTarget),
        target: relationship.target,
        host: relationship.host,
        type: relationship.activity.type,
        tags,
        dataBrokers
    });
};

const processTimeline = (timeline) => {
    const convertTimelineStatus = {
        PROGRESS: "In Progress",
        DONE: "Success",
        FAILED: "Failed",
        INTERRUPTED: "Interrupted",
        ABORTING: "Aborting",
        ABORTED: "Aborted"
    };
    timeline.createdAt = moment(timeline.createdAt).format('MMMM Do YYYY, h:mm a');
    timeline.status = convertTimelineStatus[timeline.status];
};

const attachDataBrokerUI = (dataBroker, isRunning) => {
      dataBroker = _.cloneDeep(dataBroker);
    _.set(dataBroker, 'ui.transferRate', isRunning && dataBroker.isReady ? calcTransferRate(dataBroker.transferRate) : "");
    switch (dataBroker.type) {
        case consts.FM_AGENT_TYPE_AZURE.name:
             _.set(dataBroker, 'ui.type', consts.FM_AGENT_TYPE_AZURE);
             break;
        case consts.FM_AGENT_TYPE_GCP.name:
             _.set(dataBroker, 'ui.type', consts.FM_AGENT_TYPE_GCP);
            break;
        case consts.FM_AGENT_TYPE_ONPREM.name:
             _.set(dataBroker, 'ui.type', consts.FM_AGENT_TYPE_ONPREM);
            break;
        case consts.FM_AGENT_TYPE_AWS.name:
        default:
            _.set(dataBroker, 'ui.type', consts.FM_AGENT_TYPE_AWS);
            break;
    }
    return dataBroker;
};

const calcTransferRate = (transferRate) => {
    return _.isNil(transferRate) ? "" : `(${formatBytes(transferRate).size}${formatBytes(transferRate).suffix}/s)`;
};

const processAgent = (entry, wizardType, isRunning) => {
    entry.isFailed = entry.status === consts.AGENT_STATUS_FAILED;
    entry.isWaiting = _.isEmpty(entry.placement);
    entry.isReady = !entry.isWaiting && !entry.isFailed;
    entry.isDisabled = (entry.placement && !entry.placement.awsAccountId && _.includes(wizardType, 's3'));
    entry.creationFrom = fromNowNoFuture(entry.createdAt);
    entry = attachDataBrokerUI(entry, isRunning);

    if (entry.isFailed && entry.message) entry.agentFailureMessage = entry.message.message ? entry.message.message : entry.message;

    return entry;
};

function fillWizardData(state) {
    if (!state._wizard) return state;
    const {_routing: {decodedParams}} = state;
    // console.log(`########### decodedParams from router ${JSON.stringify(decodedParams)}`);
    return state.update("_wizard", _wizard => _wizard.set("urlParams", Object.assign({}, _.get(_wizard, "urlParams"), decodedParams)));
}

const bootIntercom = (state, noUser = false) => {
    if (!_.get(state, "_config.succeeded")) {
        console.log(`%c bootIntercom config not succeeded not booting intercom ${JSON.stringify(state)}`, "font-weight: bold;");
        return;
    }

    const auth = state.auth ? state.auth.data : null;
    const _config = state._config.data;

    if (!auth || noUser) {
        console.log(`%c bootIntercom Booting with unknown user`, "font-weight: bold;");
        window.Intercom("boot", {
            app_id: _config.intercomAppId
        });
    }
    else {
        console.log(`%c bootIntercom Booting with known user ${auth.email}`, "font-weight: bold;");
        window.Intercom("boot", {
            app_id: _config.intercomAppId,
            user_id: auth.uid,
            user_hash: auth.hash,
            name: auth.name,
            email: auth.email
        });
    }
};

/*eslint-disable */
const bootGoogleAnalytics = () => {
    (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
            (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
        m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
    })(window, document, 'script', 'https://www.google-analytics.com/analytics.js', 'ga');

    window.ga('create', 'UA-82958412-15', 'auto');
    window.ga('send', 'pageview'); // see also history.listen in index.jsx
};

const bootFacebookAnalytics = () => {
    !function(f,b,e,v,n,t,s){if(f.fbq)return;n=f.fbq=function(){n.callMethod?
        n.callMethod.apply(n,arguments):n.queue.push(arguments)};if(!f._fbq)f._fbq=n;
        n.push=n;n.loaded=!0;n.version='2.0';n.queue=[];t=b.createElement(e);t.async=!0;
        t.src=v;s=b.getElementsByTagName(e)[0];s.parentNode.insertBefore(t,s)}(window,
        document, 'script', 'https://connect.facebook.net/en_US/fbevents.js');

    window.fbq('init', '1781776865442633');
    window.fbq('track', 'PageView');// see also history.listen in index.jsx
};

const bootLinkedInAnalytics = () => {
    global._linkedin_data_partner_id = "35143";
    (function () {
        const s = document.getElementsByTagName("script")[0];
        const b = document.createElement("script");
        b.type = "text/javascript";b.async = true;
        b.src = "https://snap.licdn.com/li.lms-analytics/insight.min.js";
        s.parentNode.insertBefore(b, s);
    })();
    window.lin = () => {
        const url = window.location.href;
        const queryParams = {
            time: Date.now(),
            pid: global._linkedin_data_partner_id,
            url,
            pageUrl: url,
            ref: undefined,
            fmt: 'js',
            s: 1
        };
        const img = new Image();
        img.src = `https://dc.ads.linkedin.com/collect/?${qs.stringify(queryParams)}`;
    };
};
/*eslint-enable */


const handleAction = ((state, action) => {
    action = _.cloneDeep(action);

    switch (action.type) {
        case actionCreatorsConsts.UPDATE_URL_PARAMS: {
            return state.set("_routing", action.payload);
        }
        case actionCreatorsConsts.LOGIN_LOCAL: {
            let updatedState = defaultState(state._config, state._routing).set('auth', buildStateFromAction(action))
                .setIn(['auth', 'loginType'], action.type);


            if (action.status === actionCreatorsConsts.STATUS_SUCCESS) {
                updatedState = updatedState.setIn(['auth', 'data', 'subscriptionStatus'], action.payload && action.payload.subscriptionStatus);

                updatedState = licenseUtils.licenseEnforcement(updatedState);

                bootIntercom(updatedState);
            }
            else if (action.status === actionCreatorsConsts.STATUS_ERROR) {
                updatedState = updatedState.setIn(['auth', 'loginError'], action.payload.data.message);
            }
            return updatedState;
        }
        case actionCreatorsConsts.CONFIG: {
            const updatedState = updateKeyWithBuiltState(state, action, "_config");
            const {_routing} = state;

            if (action.status === actionCreatorsConsts.STATUS_SUCCESS) {
                const force = (_.get(_routing, ["location","query", "analytics"], "")).toLowerCase() === 'true';
                if (force || updatedState._config.data.environment === 'production') {
                    bootGoogleAnalytics();
                    bootFacebookAnalytics();
                    bootLinkedInAnalytics();
                }
                bootIntercom(updatedState);
            }

            return updatedState;
        }
        case actionCreatorsConsts.VISITED_TRIAL_PAGE: {
            return state.setIn(["_ui", "visitedTrialPage"], true)
                .setIn(["_ui", "showTrialPage"], false);
        }
        case actionCreatorsConsts.SERVER_CREDENTIALS_STEP: {
            const {payload: {serverType }} = action;
            return state.setIn(["_ui", serverType, "step"], "credentials")
        }
        case actionCreatorsConsts.SERVER_IP_STEP: {
            const {payload: {serverType }} = action;
            return state.setIn(["_ui", serverType, "step"], "ip")
        }
        case actionCreatorsConsts.BUCKETS_ENCRYPTION_STEP: {
            return state.setIn(["_ui", "s3", "step"], "encryption")
        }
        case actionCreatorsConsts.AZURE_ADDITIONAL_SETTINGS_STEP: {
            return state.setIn(["_ui", "azure", "step"], "additionalSettings")
        }
        case actionCreatorsConsts.AZURE_SELECT_STEP: {
            return state.setIn(["_ui", "azure", "step"], "select")
        }

        case actionCreatorsConsts.SCROLL_TO_TOP: {
            return state.setIn(["_ui", "scrollToTop"], true)
        }
        case actionCreatorsConsts.STOP_SCROLL_TO_TOP: {
            return state.setIn(["_ui", "scrollToTop"], false)
        }
        case actionCreatorsConsts.BUCKETS_SELECT_STEP: {
            return state.setIn(["_ui", "s3", "step"], "select")
        }
        default: {
            //do nothing
        }
    }

    if (!state.auth) return state;

    switch (action.type) {
        case actionCreatorsConsts.LOGOUT: {
            window.Intercom("shutdown");
            setTimeout(() => bootIntercom(state, true), 100);//boot intercom without a user
            window.auth.logout();
            return defaultState(state._config, state._routing);
        }
        case actionCreatorsConsts.OPEN_DIALOG: {
            return state.set("dialog", action.payload)
        }
        case actionCreatorsConsts.CLOSE_DIALOG: {
            return state.set("dialog", null)
        }
        case actionCreatorsConsts.PUSH_UI_STATE: {
            return state.updateIn(["_ui", "states"], states => states ? states.concat(action.payload) : [action.payload])
        }
        case actionCreatorsConsts.POP_UI_STATE: {
            return state.updateIn(["_ui", "states"], states => _.dropRight(states, 1))
        }
        case actionCreatorsConsts.CLOSE_SEARCH_WIDGET: {
            return state.setIn(["_ui", "closeSearchWidget"], true);
        }
        case actionCreatorsConsts.CLEAR_CLOSE_SEARCH_WIDGET: {
            return state.setIn(["_ui", "closeSearchWidget"], false);
        }
        case actionCreatorsConsts.CLEAR_URL_PARAMS: {
            const {payload: {changedParam, newParamVal, objKey, without, forceClearDueToCredentialsChange }} = action;
            //if the new parameter value !== old parameter value (in urlParams) then clear what is requested:
            const oldParam = decodeAll(state._wizard.urlParams[changedParam]);
            if (forceClearDueToCredentialsChange || (oldParam && newParamVal !== oldParam)) {
                // console.log(`%c%c reducer clearUrlParams changedParam: ${changedParam} newParamVal ${newParamVal} objKey (${objKey}) without (${without}) forceClear ${forceClearDueToCredentialsChange}`, "font-weight:bold", "background:yellow");
                //clear all selected params
                //clear all params from action.payload.params. params need to be removed from urlParams and from _routing.decodedParams to be deleted completely
                return state.setIn(["_wizard", "urlParams"], undefined).updateIn(objKey, obj => obj? Immutable.without(obj, without) : obj);
            }
            return state;
        }
        case actionCreatorsConsts.UPDATE_USER: {
            if (action.status === actionCreatorsConsts.STATUS_SUCCESS && action.payload) {
                //merge the new auth from update user with the original auth
                return state.setIn(['auth', 'data'], _.merge({}, _.get(state, "auth.data"), buildStateFromAction(action).data));
            }
            return state;
        }
        case actionCreatorsConsts.GET_DATA_BROKERS: {
            return updateKeyWithBuiltState(state, action, "_dataBrokers", (builtState) => {
                if (action.status === actionCreatorsConsts.STATUS_SUCCESS && action.payload) {
                    builtState.data = builtState.data.map((entry) => processAgent(entry, _.get(state, "_wizard.wizardType")));
                }
                return builtState;
            });
        }
        case actionCreatorsConsts.GET_SERVERS: {
            const originalState = _.get(state, "_servers." + action.extra.serverType);
            if(originalState && action.status !== actions.STATUS_PROGRESS && originalState.id !== action.id){
                console.log(`%c handleAction discarding action @ ${action.type} since the id doesn't match the id of the same action in the store`, "font-weight: bold;");
                return state;
            }
            return state.setIn(["_servers", action.extra.serverType], buildStateFromAction(action));
        }
        case actionCreatorsConsts.GET_AZURE_STORAGE_ACCOUNTS: {
            return updateKeyWithBuiltState(state, action, "_azureStorageAccounts");
        }
        case actionCreatorsConsts.GET_GCP_PROJECTS: {
            return updateKeyWithBuiltState(state, action, "_gcpProjects");
        }
        case actionCreatorsConsts.DATA_BROKER: {
            return updateKeyWithBuiltState(state, action, "_createDataBroker");
        }
        case actionCreatorsConsts.DELETE_DATA_BROKER: {
            return updateKeyWithBuiltState(state, action, "_deleteDataBroker");
        }
        case actionCreatorsConsts.SYNC_NOW: {
            if (action.extra.relationshipId) {
                return state.updateIn(["_relationships", "data"], relationships => {
                    return relationships.map(relationship => {
                        if (relationship.id === action.extra.relationshipId) {
                            relationship = relationship.asMutable({deep: true});
                            relationship.activity = setActivityInProgress(relationship.activity);
                        }
                        return relationship;
                    });
                });
            }
            return updateKeyWithBuiltState(state, action, "_syncNow");
        }
        case actionCreatorsConsts.MESSAGES: {
            const messages = {
                failed: action.status === actionCreatorsConsts.STATUS_ERROR
            };
            return state.set("_messages", messages);
        }
        case actionCreatorsConsts.UPDATE_AGENT_STATUS: {
            const {status, message, id, placement} = action.payload[consts.PAYLOAD_DATA_BROKER_STATUS];
            let newState = state;

            if (_.get(state, "_relationships.data")) {
                newState = state.updateIn(["_relationships" ,"data"], relationships => {
                    return relationships.map((relationship)=>{
                        const clonedRelationship = _.cloneDeep(relationship);
                        clonedRelationship.group.dataBrokers = clonedRelationship.group.dataBrokers.map((dataBroker) => {
                            if(dataBroker.id === id) {
                                if (status) dataBroker.status = status;
                                if (placement) dataBroker.placement = placement;
                                if (message) dataBroker.message = message;
                            }
                            return dataBroker;
                        });
                        processRelationship(clonedRelationship);
                        return clonedRelationship;
                    });
                });
            }

            if (!_.get(state, "_dataBrokers.data")) {
                return newState;
            }

            newState = state.updateIn(["_dataBrokers", "data"], dataBrokers => {
                return dataBrokers.map(dataBroker => {
                    if (dataBroker.id === id) {
                        dataBroker = _.cloneDeep(dataBroker);
                        if (status) dataBroker.status = status;
                        if (placement) dataBroker.placement = placement;
                        if (message) dataBroker.message = message;

                        processAgent(dataBroker, _.get(state, "_wizard.wizardType"));
                    }
                    return dataBroker;
                });
            });
            return newState;
        }
        case actionCreatorsConsts.UPDATE_RELATIONSHIP: {
            const {id, activity, group} = action.payload[consts.PAYLOAD_UPDATE_RELATIONSHIP];

            let newUpdatedState;
            let isNewRelationship = true;

            if (_.get(state, "_relationships.data")) {
                newUpdatedState = state.updateIn(["_relationships", "data"], relationships => {
                    return relationships.map(relationship => {
                        if (relationship.id === id) {
                            isNewRelationship = false;
                            const thisRequest = _.find(relationship.lastRequests, function (req) {
                                return req.id === action.payload.requestId;
                            });
                            if (thisRequest && thisRequest.isDone) return relationship; //dont get more update for this request after done status
                            else if (relationship.activity.status === consts.RESPONSE_STATUS_ABORTING && activity.status === consts.RESPONSE_STATUS_RUNNING) {
                                //if in aborting status, dot update to syncing
                                return relationship;
                            }
                            const clonedRelationship = _.cloneDeep(relationship);
                            clonedRelationship.activity = activity;
                            clonedRelationship.group = group;
                            processRelationship(clonedRelationship);
                            return clonedRelationship;
                        }
                        return relationship;
                    })
                });
            }

            if (isNewRelationship) {
                newUpdatedState = state.update("_ui", _ui => _ui.set("hasNewRelationship", true));
            }
            newUpdatedState = updateLastRequestsForRelationshipsData(newUpdatedState, action);

            //update only if sync already exists, hence we are in the wizard
            if (_.get(newUpdatedState, '_wizard.sync') && _.get(newUpdatedState, '_wizard.sync.relationshipId') === id) {
                if (activity.status === consts.RESPONSE_STATUS_FAILED) {
                    return newUpdatedState.updateIn(["_wizard", "sync"], sync => sync.set("completed", false).set("failed", true));
                } else if (activity.status === consts.RESPONSE_STATUS_DONE) {
                    return newUpdatedState.updateIn(["_wizard", "sync"], sync => sync.set("completed", true).set("percentCopied", 100));
                } else {
                    return newUpdatedState.updateIn(["_wizard", "sync"], sync => sync.set("percentCopied", activity.progress).set("completed", false));
                }
            }
            return newUpdatedState;
        }
        case actionCreatorsConsts.UPDATE_SUBSCRIPTION_STATUS: {
            const builtState = state.setIn(['auth', 'data', 'subscriptionStatus'], action.payload[consts.PAYLOAD_UPDATE_SUBSCRIPTION_STATUS].subscriptionStatus);

            return licenseUtils.licenseEnforcement(builtState);
        }
        case actionCreatorsConsts.UPDATE_DATABROKER_TRANSFER_RATE: {
            const {dataBrokerId, transferRate} = action.payload[consts.PAYLOAD_DATABROKER_TRANSFER_RATE];

            if (!_.get(state, "_relationships.data")) {
                return state;
            }

            return state.updateIn(["_relationships", "data"], relationships => {
                return relationships.map(relationship => {
                    if (relationship.activity.status !== consts.RESPONSE_STATUS_RUNNING) {
                        return relationship;
                    }

                    const clonedRelationship = relationship.asMutable({deep: true});
                    if (clonedRelationship.group && clonedRelationship.group.dataBrokers) {
                        clonedRelationship.group.dataBrokers.forEach((dataBroker) => {
                            if(dataBroker && dataBroker.id === dataBrokerId){
                                if (transferRate) {
                                    dataBroker.transferRate = transferRate;
                                    processRelationship(clonedRelationship);
                                }
                            }
                        })
                   }
                   return clonedRelationship;
                });
            });

        }
        case actionCreatorsConsts.UPDATE_SYNC_SCHEDULE: {
            if (!_.get(state, "_relationships.data")) {
                return addUnhandledMessages(state, action);
            }
            if (action.status === actionCreatorsConsts.STATUS_SUCCESS && action.payload) {
                return state.updateIn(["_relationships", "data"], relationships => {
                    return relationships.map(relationship => {
                        if (relationship.id === action.payload.id) {
                            relationship = relationship.asMutable({deep: true});
                            relationship.autoSync = action.payload.autoSync;
                            relationship.sync = action.payload.sync;
                            processRelationship(relationship);
                        }
                        return relationship;
                    });
                });
            }
            return state;
        }
        case actionCreatorsConsts.DELETE_RELATIONSHIP: {
            if (!_.get(state, "_relationships.data")) {
                return addUnhandledMessages(state, action);
            }
            if (action.extra.id) {
                return state.updateIn(["_relationships", "data"], relationships => {
                    return relationships.map(relationship => {
                        if (relationship.id === action.extra.id) {
                            relationship = relationship.asMutable({deep: true});
                            relationship.activity.isBeingDeleted = true;
                            processRelationship(relationship);
                        }
                        return relationship;
                    });
                });
            }
            return state;
        }
        case actionCreatorsConsts.ABORT_SYNC: {
            if (!_.get(state, "_relationships.data")) {
                return addUnhandledMessages(state, action);
            }
            if (action.extra.relationshipId) {
                return state.updateIn(["_relationships", "data"], relationships => {
                    return relationships.map(relationship => {
                        if (relationship.id === action.extra.relationshipId) {
                            relationship = relationship.asMutable({deep: true});
                            relationship.activity.status = consts.RESPONSE_STATUS_ABORTING;
                            processRelationship(relationship);
                        }
                        return relationship;
                    });
                });
            }
            return state;
        }
        case actionCreatorsConsts.GET_RELATIONSHIPS: {
            let nextState = updateKeyWithBuiltState(state, action, "_relationships", (builtState) => {
                if (action.status === actionCreatorsConsts.STATUS_PROGRESS && !state._relationships) builtState.firstTime = true;

                if (action.status === actionCreatorsConsts.STATUS_SUCCESS && action.payload) {

                    builtState.data.map((entry) => {
                        entry.lastCopyMessage = entry.lastCopyMessage || {};
                        entry.lastScanMessage = entry.lastScanMessage || {};
                        entry.lastSyncMessage = entry.lastSyncMessage || undefined;
                        entry.lastRequests = entry.lastRequests || [];

                        //Asterix support: "group" doesn't exist, so we manually put the single data broker in the group array
                        if (entry.dataBroker && !entry.dataBroker.isNewDataBroker && !entry.group) {
                            entry.group = {dataBrokers: [entry.dataBroker]};
                        }

                        processRelationship(entry);

                        return entry;
                    });
                }
                return builtState;
            });

            if (action.status === actionCreatorsConsts.STATUS_SUCCESS && action.payload) {
                nextState = licenseUtils.licenseEnforcement(nextState);
            }

            return nextState.update("_ui", _ui => _ui.set("hasNewRelationship", false));
        }
        case actionCreatorsConsts.START_WIZARD: {
            action.payload.sync = {};
            const wizardType = action.payload.name;
            action.payload.stepNames = action.payload.stepNamesByWizardType[wizardType];
            action.payload.wizardType = wizardType;
            action.payload.sourceType = getEndPointType(wizardType, "source");
            action.payload.targetType = getEndPointType(wizardType, "target");

            return state.set("_wizard", action.payload);
        }
        case actionCreatorsConsts.STARTED_WIZARD_STEP: {
            return updateCurrentStep(state.updateIn(["_wizard", "steps"], steps => (steps || Immutable({})).set(action.payload, {started: true})), action.payload, true);
        }
        case actionCreatorsConsts.EXITED_WIZARD_STEP: {
            return updateErrorsPanelStatus(updateCurrentStep(state.updateIn(["_wizard", "steps"], steps => steps.without(action.payload)), action.payload, false).updateIn(["_generalErrors", "_errorsMap"], () => ({})));
        }
        case actionCreatorsConsts.CLEAR_SYNC: {
            return state.update("_wizard", _wizard => _wizard.set("sync", {}));
        }
        case actionCreatorsConsts.CLEAR_WIZARD_CACHE: {
            const {cifs, sgws, s3, azure, gcp} = action.payload;
            //clear credentials from local storage
            if (cifs) removeCredentialsByType("cifs");
            if (sgws) removeCredentialsByType("objectStorage");
            if (s3) removeCredentialsByType("s3");
            if (azure) removeCredentialsByType("azure");
            if (gcp) removeCredentialsByType("gcp");

            removeSettingsFromSessionStorage();

            //clear cache of wizard as well
            return state.set("_showBuckets", undefined)
                .set("_showExports", undefined)
                .set("_showShares", undefined)
                .set("_showSGBuckets", undefined)
                .set("_showGCPBuckets", undefined)
                .set("_gcpProjects", undefined)
                .setIn(["_routing", "decodedParams"], undefined)
                .set("_servers", undefined)
                .set("_azureStorageAccounts", undefined)
                .set("_showAzureBlobContainers", undefined)
                .set("_dataBrokers", undefined);
        }
        case actionCreatorsConsts.ADD_BUCKET_NAME: {

            const {bucketName, dataBrokerId} = action.payload;
            if (bucketName) {
                const bucketObject = {
                    actualPath: bucketName,
                    name: bucketName,
                    path: bucketName,
                    wasAddedManually: true,
                    region: {displayName: "Unknown", name: "Unknown"},
                    tags: []
                };
                const addNewBucket = (data) => {
                    var dataMutable = data ? data.asMutable() : [];
                    dataMutable.unshift(bucketObject);
                    return Immutable(dataMutable);
                };
                if (!state._showBuckets[dataBrokerId].data.find(bucket => bucket.name === bucketName.split('/')[0])) { //if bucket not in the list
                    return state.updateIn(["_showBuckets", dataBrokerId, "folders", "-", "data"], data => addNewBucket(data))
                        .updateIn(["_showBuckets", dataBrokerId, "data"], data => addNewBucket(data));
                }
            }
            return state;
        }
        case actionCreatorsConsts.ADD_EXPORT_MANUALLY: {
            const {exportName, nfsServer} = action.payload;

            if (exportName) {
                const trimmedExport = exportName[exportName.length - 1] === "/" ? exportName.substring(0, exportName.length - 1) : exportName;
                const exportObject = {
                    actualPath: trimmedExport,
                    path: trimmedExport,
                    host: nfsServer
                };
                const addNewExport = (data) => {
                    var dataMutable = data ? data.asMutable() : [];
                    dataMutable.unshift(exportObject);
                    return Immutable(dataMutable);
                };
                if (!(state._showExports[nfsServer].data && state._showExports[nfsServer].data.find(exp => exp.path === trimmedExport))) { //if export not in the list
                    return state.updateIn(["_showExports", nfsServer, "folders", "-", "data"], data => addNewExport(data))
                        .updateIn(["_showExports", nfsServer, "data"], data => addNewExport(data))
                        .updateIn(["_generalErrors", "_errorsMap"], () => ({}))
                        .updateIn(["_generalErrors", "_errorsPanelSize"], () => ({height: 0}));
                }
            }
            return state;
        }
        case actionCreatorsConsts.CREATE_SYNC: {
            const updatedState = state.update("_wizard", _wizard => _wizard.set("sync", buildStateFromAction(action)));
            if (action.status === actionCreatorsConsts.STATUS_SUCCESS) {
                //send event to GA and facebook
                sendEvents('relationship', updatedState.auth.data.email, "created_relationship");
                return updatedState.updateIn(["_wizard", "sync"], sync => sync.set("completed", false).set("percentCopied", 0).set("bytesCopied", 0).set("relationshipId", action.payload.id))
                    .update("_wizard", _wizard => _wizard.set("hideHeader", true));
            }
            else return updatedState;
        }
        case actionCreatorsConsts.SHOW_EXPORTS: {
            return listActionsUtils.updateStoreForShowAction(["_showExports", encodeAll(action.extra.host)], ['host', 'dataBrokerId'], state, action);
        }
        case actionCreatorsConsts.UPDATE_EXPORTS: {
            const host = action.payload[consts.PAYLOAD_SHOW_EXPORTS].host;
            const fullKey = ["_showExports", encodeAll(host)];
            return listActionsUtils.updateStoreForUpdateShowActionForFileServer(fullKey, consts.PAYLOAD_SHOW_EXPORTS, 'export', state, action, listActionsUtils.isHostEqual.bind(this, fullKey, consts.PAYLOAD_SHOW_EXPORTS, 'export', state, action));

        }
        case actionCreatorsConsts.SHOW_SHARES: {
            return listActionsUtils.updateStoreForShowAction(["_showShares", encodeAll(action.extra.host)], ['host', 'dataBrokerId', 'domain', 'username', 'password'], state, action);
        }
        case actionCreatorsConsts.UPDATE_SHARES: {
            const host = action.payload[consts.PAYLOAD_LIST_CIFS_SHARES].host;
            const fullKey = ["_showShares", encodeAll(host)];
            return listActionsUtils.updateStoreForUpdateShowActionForFileServer(fullKey, consts.PAYLOAD_LIST_CIFS_SHARES, "share", state, action, listActionsUtils.isHostEqual.bind(this, fullKey, consts.PAYLOAD_LIST_CIFS_SHARES, "share", state, action));

        }
        case actionCreatorsConsts.LIST_FOLDERS: {
            const fullKey = ["_showExports", encodeAll(action.extra.fileServer)];
            return listActionsUtils.updateStoreForListFoldersAction(fullKey, action.extra.path, state, action);

        }
        case actionCreatorsConsts.UPDATE_FOLDERS: {
            const fullKey = ["_showExports", encodeAll(action.payload[consts.PAYLOAD_LIST_FOLDERS].nfsServer)];
            return listActionsUtils.updateStoreForUpdateFoldersActionForFileServer(fullKey, consts.PAYLOAD_LIST_FOLDERS, state, action);

        }
        case actionCreatorsConsts.LIST_SHARE_FOLDERS: {
            const fullKey = ["_showShares", encodeAll(action.extra.host)];
            return listActionsUtils.updateStoreForListFoldersAction(fullKey, action.extra.path, state, action);

        }
        case actionCreatorsConsts.UPDATE_SHARE_FOLDERS: {
            const fullKey = ["_showShares", encodeAll(action.payload[consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS].host)];
            return listActionsUtils.updateStoreForUpdateFoldersActionForFileServer(fullKey, consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS, state, action);

        }
        case actionCreatorsConsts.SHOW_BUCKETS: {
            return listActionsUtils.updateStoreForShowAction(["_showBuckets", action.extra.dataBrokerId], [], state, action);
        }
        case actionCreatorsConsts.UPDATE_BUCKETS: {
            return listActionsUtils.updateStoreForUpdateBucketsAction(["_showBuckets", action.payload.dataBrokerId], consts.PAYLOAD_LIST_BUCKETS, null, state, action);
        }
        case actionCreatorsConsts.LIST_BUCKETS_FOLDERS: {
            return listActionsUtils.updateStoreForListFoldersAction(["_showBuckets", action.extra.dataBrokerId], action.extra.path, state, action, listActionsUtils.isObjExistInField.bind(this, ["_showBuckets", action.extra.dataBrokerId], state, {name: action.extra.bucket}));
        }
        case actionCreatorsConsts.UPDATE_BUCKETS_FOLDERS: {
            return listActionsUtils.updateStoreForUpdateBucketsFoldersAction(["_showBuckets", action.payload.dataBrokerId], consts.PAYLOAD_LIST_SUB_DIRECTORIES, state, action);
        }
        case actionCreatorsConsts.SHOW_AZURE_BLOB_CONTAINERS: {
            return listActionsUtils.updateStoreForShowAction(["_showAzureBlobContainers", action.extra.storageAccountName], [], state, action);
        }
        case actionCreatorsConsts.UPDATE_CONTAINERS: {
            return listActionsUtils.updateStoreForUpdateBucketsAction(["_showAzureBlobContainers", action.payload[consts.PAYLOAD_LIST_CONTAINERS].storageAccountName], consts.PAYLOAD_LIST_CONTAINERS, "containers", state, action);
        }
        case actionCreatorsConsts.LIST_CONTAINERS_FOLDERS: {
            return listActionsUtils.updateStoreForListFoldersAction(["_showAzureBlobContainers", action.extra.storageAccountName], action.extra.path, state, action, listActionsUtils.isObjExistInField.bind(this, ["_showAzureBlobContainers", action.extra.storageAccountName], state, {name: action.extra.container}));
        }
        case actionCreatorsConsts.UPDATE_CONTAINERS_FOLDERS: {
            return listActionsUtils.updateStoreForUpdateBucketsFoldersAction(["_showAzureBlobContainers", action.payload[consts.PAYLOAD_LIST_CONTAINER_SUB_DIRECTORIES].storageAccountName], consts.PAYLOAD_LIST_CONTAINER_SUB_DIRECTORIES, state, action, "container", "folders");
        }

        case actionCreatorsConsts.SHOW_SG_BUCKETS: {
            return listActionsUtils.updateStoreForShowAction(["_showSGBuckets", action.extra.host], ["host", "dataBrokerId", "port", "accessKey", "secretKey"], state, action);
        }
        case actionCreatorsConsts.UPDATE_SG_BUCKETS: {
            const theHost = action.payload.data ? (action.payload.data.host ? action.payload.data.host : action.payload.data[consts.PAYLOAD_LIST_SGWS_BUCKETS].host) : action.payload[consts.PAYLOAD_LIST_SGWS_BUCKETS].host;
            return listActionsUtils.updateStoreForUpdateBucketsAction(["_showSGBuckets", theHost], consts.PAYLOAD_LIST_SGWS_BUCKETS, "buckets", state, action);
        }
        case actionCreatorsConsts.LIST_SG_BUCKETS_FOLDERS: {

            return listActionsUtils.updateStoreForListFoldersAction(["_showSGBuckets", action.extra.host], action.extra.path, state, action, listActionsUtils.isObjExistInField.bind(this, ["_showSGBuckets", action.extra.host], state, {name: action.extra.bucket}));
        }
        case actionCreatorsConsts.UPDATE_SG_BUCKETS_FOLDERS: {
            const relHost = action.payload.data ? (action.payload.data.host ? action.payload.data.host : action.payload.data[consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS].host) : action.payload[consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS].host;
            return listActionsUtils.updateStoreForUpdateBucketsFoldersAction(["_showSGBuckets", relHost], consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS, state, action);
        }

        case actionCreatorsConsts.SHOW_GCP_BUCKETS: {
            const {projectId} = action.extra;
            return listActionsUtils.updateStoreForShowAction(["_showGCPBuckets", projectId], ["projectId", "dataBrokerId", "clientEmail", "privateKey"], state, action);
        }
        case actionCreatorsConsts.UPDATE_GCP_BUCKETS: {
            return listActionsUtils.updateStoreForUpdateBucketsAction(["_showGCPBuckets", action.payload[consts.PAYLOAD_LIST_GCP_BUCKETS].projectId], consts.PAYLOAD_LIST_GCP_BUCKETS, "buckets", state, action);
        }
        case actionCreatorsConsts.LIST_GCP_BUCKET_FOLDERS: {
            const { projectId } = action.extra;
            return listActionsUtils.updateStoreForListFoldersAction(["_showGCPBuckets", projectId], action.extra.path, state, action, listActionsUtils.isObjExistInField.bind(this, ["_showGCPBuckets", projectId], state, {name: action.extra.bucket}));
        }
        case actionCreatorsConsts.UPDATE_GCP_BUCKET_FOLDERS: {
            return listActionsUtils.updateStoreForUpdateBucketsFoldersAction(["_showGCPBuckets", action.payload[consts.PAYLOAD_LIST_GCP_BUCKET_FOLDERS].projectId], consts.PAYLOAD_LIST_GCP_BUCKET_FOLDERS, state, action);
        }

        case actionCreatorsConsts.GET_TIMELINE: {
            return updateKeyWithBuiltState(state, action, "_timeline", (builtState) => {
                if (action.status === actionCreatorsConsts.STATUS_SUCCESS && action.payload) {
                    builtState.data.map((timeline) => {
                        processTimeline(timeline);
                        return timeline;
                    })
                }

                return builtState;
            });
        }
        case actionCreatorsConsts.UPDATE_TIMELINE: {
            let actionTimeline = action.payload[consts.PAYLOAD_TIMELINE];
            let updatedTimelineState = state;
            if (state._timeline.data) {
                if (!_.find(state._timeline.data, (o) => //if same timeline row is not in the table already, and this relatioship requestId didnt get done status
                    {return (o.id === actionTimeline.id || (o.requestId === actionTimeline.requestId && o.status === "Success"))})) {
                    processTimeline(actionTimeline);
                    updatedTimelineState = state.updateIn(["_timeline", "data"], timelineActions => {
                        if (timelineActions) {
                            var actionsMutable = timelineActions.asMutable();
                            actionsMutable.unshift(actionTimeline);
                            return Immutable(actionsMutable);
                        }
                        return timelineActions;
                    });
                }

                else {
                    updatedTimelineState = state.updateIn(["_timeline", "data"], timelines => {
                        return timelines.map(timeline => {
                            if (timeline.id === actionTimeline.id) {
                                timeline = actionTimeline;
                                processTimeline(timeline);
                            }
                            return timeline;
                        });
                    })
                }
            }
            return updatedTimelineState;
        }
        case actionCreatorsConsts.GET_LICENSE_SETTINGS: {
            const calculateLicenseType = ({freeTrial, totalReservedRelationships, chargedRelationships}) => {
                if (freeTrial) return consts.LICENSE_TYPE_FREE_TRIAL;
                else {
                    if (Number.isInteger(totalReservedRelationships)) {
                        if (totalReservedRelationships === 0) {
                            return consts.LICENSE_TYPE_PAYGO;
                        }
                        else if (chargedRelationships > 0) {
                            return consts.LICENSE_TYPE_PAYGO_WITH_BYOL;
                        }
                        else {
                            return consts.LICENSE_TYPE_BYOL
                        }
                    }
                }
                return consts.LICENSE_TYPE_UNKNOWN;
            };

            let builtState = updateKeyWithBuiltState(state, action, "_licenseSettings");

            if (action.status === actionCreatorsConsts.STATUS_SUCCESS && action.payload) {
                builtState = licenseUtils.licenseEnforcement(builtState);

                return builtState.setIn(["_licenseSettings", "data", "licenseType"], calculateLicenseType(action.payload));
            }

            return builtState;
        }
        case actionCreatorsConsts.GET_BYOL_LICENSES: {
            return licenseUtils.licenseEnforcement(updateKeyWithBuiltState(state, action, "_licenses"));
        }
        case actionCreatorsConsts.ADD_BYOL_LICENSE: {
            if (action.status === actionCreatorsConsts.STATUS_SUCCESS && action.payload) {
                return state.updateIn(["_licenses", "data", "licenses"], licenses => {
                    var licensesMutable = licenses ? licenses.asMutable() : [];
                    licensesMutable.push(action.payload);
                    return Immutable(licensesMutable);
                });
            }
            return state;
        }
        case actionCreatorsConsts.TOGGLE_ERRORS_STATE: {
            return state.updateIn(["_generalErrors", "_errorsPanelOpen"], errorsPanelOpen => !errorsPanelOpen);
        }
        case actionCreatorsConsts.REMOVE_ERROR: {
            return updateErrorsPanelStatus(state.updateIn(["_generalErrors", "_errorsMap"], errors => errors.without(action.payload)));
        }
        case actionCreatorsConsts.BROADCAST_ERRORS_PANEL_SIZE: {
            return state.updateIn(["_generalErrors"], generalErrors => generalErrors.set("_errorsPanelSize", {height: action.payload.height}));
        }
        case actionCreatorsConsts.CLEAR_CONTENT_HEIGHT: {
            return state.setIn(["_ui", "clearContentHeight"], true);
        }
        case actionCreatorsConsts.GET_COST_TIMES: {
            return updateKeyWithBuiltState(state, action, "_costTimes", (builtState) => {
                if (action.status === actionCreatorsConsts.STATUS_SUCCESS && action.payload) {
                    builtState.data.map((time) => {
                        const stringTime = costTime(time.month, time.year);
                        time.title = stringTime;
                        time.label = stringTime;
                        time.value = stringTime;
                        return time;
                    });
                }
                return builtState;
            });
        }
        case actionCreatorsConsts.GET_COST: {
            if (action.status !== actionCreatorsConsts.STATUS_SUCCESS || !action.payload) return state;
            const {year, month} = action.payload;
            const stringTime = costTime(month, year);
            const builtState = buildStateFromAction(action);
            return state.setIn(["_cost", stringTime], builtState);
        }
        default:
            return state;
    }
});

const handleMessageWithLog = (state, action) => {
    console.log(`%c Handling unhandled message @ ${action.type} (${action.unhandledCounter})`, "font-weight: bold;");
    return handleAction(state, action);
};

const handleUnhandledMessages = (state => {
    return state.unhandledMessages.reduce(handleMessageWithLog, state.set("unhandledMessages", []));
});


export default (history) => combineReducers({
    //handle update messages only in dashboard and review step, to optimize the reducer
    syncReducer: ignoreActions(syncReducer, (action) => ((action.type === actionCreatorsConsts.UPDATE_RELATIONSHIP) && !(window.location.href.includes("dashboard") || window.location.href.includes("review")))),
    router: connectRouter(history),
    form: formReducer
});
