import axios from 'axios';
import _ from "lodash";
import * as consts from "./consts";

axios.defaults.baseURL = '/api';
axios.defaults.headers.post['Content-Type'] = 'application/json';
axios.defaults.headers.put['Content-Type'] = 'application/json';

const auth = (t) => {
    axios.defaults.headers.common['Authorization'] = t.authorization;
    return t;
};

const extractData = (response) => {
    const data = response.data;
    const requestId = response.headers["x-dfio-req-id"];
    if (data && !_.isString(data)) {
        data.requestId = requestId;
        return data;
    } else return {requestId};
};

export default {
    userInfo: function (token, mpCustomerToken) {
        auth({authorization: token.token});
        if (mpCustomerToken) {
            return axios.get('/users/info', {params: {mpCustomerToken}}).then(extractData);
        } else {
            return axios.get('/users/info').then(extractData);
        }
    },
    updateUser: function (data) {
        return axios.put(`/users/local/update`, data).then(extractData);
    },
    getConfig: function () {
        return axios.get("/config").then(extractData);
    },
    createDataBroker: function (opts) {
        return axios.post('/data-brokers', opts).then(extractData);
    },
    deleteDataBroker: function (id) {
        return axios.delete(`/data-brokers/${id}`).then(extractData);
    },
    dataBrokers: function () {
        return axios.get('/data-brokers').then(extractData);
    },

    getServers: function (serverType) {
        return axios.get('/storages/' + serverType).then(extractData);
    },

    gcpProjects: function () {
        return axios.get('/storages/gcp').then(extractData);
    },
    azureStorageAccounts: function () {
        return axios.get('/storages/azure').then(extractData);
    },
    messages: function (lastMessage) {
        const filter = (lastMessage) ? '?last=' + lastMessage : '';
        return axios.get(`/messages/client${filter}`).then(extractData);
    },
    showExports: function (dataBrokerId, nfsServer) {
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_SHOW_EXPORTS}`, {
            params: {
                host: nfsServer,
            }
        }).then(extractData);
    },
    showEFSExports: function (dataBrokerId, nfsServer) {
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_SHOW_EXPORTS}?host=${nfsServer}&isEFS=${true}`).then(extractData);
    },
    listFolder: function (dataBrokerId, nfsServer, path, exportName, nfsVersion) {
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_FOLDERS}`, {
            params: {
                host: nfsServer,
                path,
                exportName,
                version: nfsVersion || undefined
            }
        }).then(() => ({dataBrokerId, nfsServer, path}));
    },
    showShares: function (dataBrokerId, cifsServer, username, password, domain, cifsVersion) {
        let headers = {
            username,
            password
        };
        if (domain) {
            headers.domain = domain;
        }

        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_CIFS_SHARES}`, {
            params: {
                host: cifsServer,
                version: cifsVersion || undefined
            },
            headers
        }).then(extractData);
    },
    listShareFolders: function (dataBrokerId, host, share, path, username, password, domain, cifsVersion) {
        let headers = {
            username,
            password
        };
        if (domain) {
            headers.domain = domain;
        }
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS}`, {
            params: {
                host,
                share,
                path,
                version: cifsVersion || undefined
            },
            headers
        }).then(extractData);
    },
    fetchSGBuckets: function (dataBrokerId, sgServer, sgCredentials) {
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_SGWS_BUCKETS}`, {
            params: {
                host: sgServer,
                port: sgCredentials.port
            },
            headers: {
                "access-key": sgCredentials.accessKey,
                "secret-key": sgCredentials.secretKey
            }
        }).then(extractData);
    },

    listSGBucketFolders: function (dataBrokerId, host, bucket, path, accessKey, secretKey, port) {
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS}`, {
            params: {
                host,
                bucket,
                prefix: path,
                port

            },
            headers: {
                "access-key": accessKey,
                "secret-key": secretKey
            }
        }).then(extractData);
    },

    fetchBuckets: function (dataBrokerId) {
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_BUCKETS}`).then(extractData);
    },

    listBucketFolders: function (dataBrokerId, bucket, prefix) {
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_SUB_DIRECTORIES}?bucket=${bucket}&prefix=${prefix}`).then(extractData);
    },

    fetchAzureBlobContainers: function (dataBrokerId, storageAccountName, connectionString) {
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_CONTAINERS}`, {
            params: {
                'storage-account-name': storageAccountName
            },
            headers: {
                'storage-account-connection-string': connectionString
            }
        }).then(extractData);
    },

    listContainerFolders: function (dataBrokerId, storageAccountName, connectionString, container, path) {
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_CONTAINER_SUB_DIRECTORIES}`, {
            params: {
                'storage-account-name': storageAccountName,
                container,
                prefix:path !== "" ? path : undefined
            },
            headers: {
                'storage-account-connection-string': connectionString
            }
        }).then(extractData);
    },

    listGCPBuckets: function (dataBrokerId, projectId, clientEmail, privateKey) {
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_GCP_BUCKETS}`, {
            params: {
                'project-id': projectId
            },
            headers: {
                'client-email': clientEmail,
                'private-key': privateKey
            }
        }).then(extractData);
    },

    listGCPBucketFolders: function (dataBrokerId, projectId, bucket, path, clientEmail, privateKey) {
        return axios.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_GCP_BUCKET_FOLDERS}`, {
            params: {
                'project-id': projectId,
                bucket,
                prefix:path !== "" ? path : undefined
            },
            headers: {
                'client-email': clientEmail,
                'private-key': privateKey
            }
        }).then(extractData);
    },

    createSync: function (opts) {
        return axios.post(`/relationships-v2`, opts).then(extractData);
    },
    syncNow: function (relationshipId) {
        return axios.put(`/relationships/${relationshipId}/sync`, {}).then(extractData);
    },
    abortSync: function (relationshipId) {
        return axios.put(`/relationships/${relationshipId}/abort`, {}).then(extractData);
    },
    getRelationships: function () {
        return axios.get("/relationships-v2").then(extractData);
    },
    deleteRelationship: function (relationshipId) {
        return axios.delete(`/relationships/${relationshipId}`).then(extractData);
    },
    updateSyncSchedule: function (relationshipId, values) {
        return axios.put(`/relationships/${relationshipId}/sync-schedule`, values).then(extractData);
    },
    updateRelationship: function (relationshipId, opts) {
        return axios.put(`/relationships-v2/${relationshipId}`, opts).then(extractData);
    },
    changeAWSId: function (data) {
        return axios.put(`/users/local/update`, data).then(extractData);
    },
    changePassword: function (data) {
        return axios.put(`/users/local/change-password`, data).then(extractData);
    },
    getTimeline: function () {
        return axios.get("/timelines").then(extractData);
    },
    getCost: function (params) {
        return axios.get("/users/cost", {params}).then(extractData);
    },
    getCostTimes: function () {
        return axios.get("/users/cost-times").then(extractData);
    },
    getLicenseSettings: function () {
        return axios.get("/users/license-settings").then(extractData);
    },
    getBYOLLicenses: function () {
        return axios.get("/users/licenses").then(extractData);
    },
    updateNSSCredentials: function (values) {
        return axios.put("/users/nss-credentials", values).then(extractData);
    },
    addBYOLLicense: function (values) {
        return axios.put("/users/licenses", values).then(extractData);
    },
    interceptors: function () {
        return axios.interceptors;
    }
};
