import _ from "lodash";
import {RESPONSE_STATUS_RUNNING, RESPONSE_STATUS_DONE, RESPONSE_STATUS_FAILED, RESPONSE_STATUS_INTERRUPTED} from "./../consts"
import numeral from "numeral";
import {formatBytes} from "./reducerUtils";
import {fromNowNoFuture, stringDuration} from "./time";


export const statusConvert = {
    SUCCESS: "Synced Successfully",
    COMPLETE: "Sync Completed",
    FAILED: "Failed To Sync",
    RUNNING: "Sync In Progress",
    INTERRUPTED: "Sync Interrupted",
    ABORTING: "Aborting Sync",
    ABORTED: "Sync Aborted"
};

export const formatActivityValues = (activity) => {
    return {
        formattedFilesCopied: numeral(activity.filesCopied).format('0.[0]a'),
        formattedFilesRemoved: !_.isNil(activity.filesRemoved) ? numeral(activity.filesRemoved).format('0.[0]a') : null,
        formattedFilesFailed: numeral(activity.filesFailed + activity.filesRemovedFailed).format('0.[0]a'),
        formattedBytesCopied: formatBytes(activity.bytesCopied),
        formattedBytesRemoved: !_.isNil(activity.bytesRemoved) ? formatBytes(activity.bytesRemoved) : null ,
        formattedBytesFailed: formatBytes(activity.bytesFailed + activity.bytesRemovedFailed),
        formattedFilesMarkedForCopy: numeral(activity.filesMarkedForCopy).format('0.[0]a'),
        formattedBytesMarkedForCopy: formatBytes(activity.bytesMarkedForCopy),
        formattedDirsMarkedForCopy: numeral(activity.dirsMarkedForCopy).format('0.[0]a'),
        formattedFilesMarkedForRemove: numeral(activity.filesMarkedforRemove).format('0.[0]a'),
        formattedBytesMarkedForRemove: formatBytes(activity.bytesMarkedForRemove),
        formattedDirsMarkedForRemove: numeral(activity.dirsMarkedForRemove).format('0.[0]a'),
        formattedDirsScanned : numeral(activity.dirsScanned).format('0.[0]a'),
        formattedDirsFailedToScan : numeral(activity.dirsFailedToScan).format('0.[0]a'),
        formattedFilesScanned : numeral(activity.filesScanned).format('0.[0]a'),
        formattedBytesScanned : formatBytes(activity.bytesScanned),
    };
};

export const processActivityForTimeline = (activity) =>{
    const stats = formatActivityValues(activity);
    const isRunning = activity.status === RESPONSE_STATUS_RUNNING;
    const lastTransferred = activity.lastMessageTime ? fromNowNoFuture(activity.lastMessageTime) : "No previous copy";
    const duration = activity.executionTime ? stringDuration(activity.executionTime) : "No previous copy";
    const lastTrasnferText = (activity.status === RESPONSE_STATUS_FAILED || activity.status === RESPONSE_STATUS_INTERRUPTED) ? lastTransferred :
        (isRunning ? `Duration: ${duration}` : `${lastTransferred} | Duration: ${duration}` );

    const res = Object.assign({}, stats, activity);
    const statusConverted = activity.status=== RESPONSE_STATUS_DONE ?
        (activity.filesFailed === 0 ? 'SUCCESS' : 'COMPLETE' ) : activity.status;
    res.statusText = statusConvert[statusConverted];
    res.duration = duration;
    res.lastTransferredHeaderText = lastTrasnferText;
    delete res.progress;
    return res;

};

export function createStringFromEndPointObject(object) {
    const {protocol} = object;
    const details = object[protocol];
    let objectString;
    switch (protocol) {
        case 'sgws':
        case 'ibm': {
            const {host, port, bucket} = details;
            let {prefix} = details;
            const fixedProtocol = details.objectStorageType || "sgws";
            objectString = (port) ? `${fixedProtocol}://${host}:${port}` : `${fixedProtocol}://${host}`;
            prefix = (prefix) ? (prefix.startsWith('/') ? prefix.substr(1) : prefix) : prefix;
            objectString = (prefix) ? `${objectString}/${bucket}/${prefix}` : `${objectString}/${bucket}`;
            break;
        }
        case 'cifs': {
            const {host} = details;
            let {path, share} = details;
            path = (path) ? (path.startsWith('/') ? path.substr(1) : path) : path;
            share = share.startsWith('/') ? share.substr(1) : share;
            objectString = (path) ? `cifs://${host}/${share}/${path}` : `cifs://${host}/${share}`;
            break;
        }
        case 'efs':
        case 'cvs':
        case 'anf':
        case 'nfs': {
            const {host} = details;
            let {path, export: nfsExport} = details;
            const fixedProtocol = (details.nfsType && details.nfsType !== "snfs") ? details.nfsType : "nfs";

            path = (path) ? (path.startsWith('/') ? path.substr(1) : path) : path;
            nfsExport = nfsExport.startsWith('/') ? nfsExport.substr(1) : nfsExport;
            const hostAndRootPath = _.isEmpty(nfsExport) ? `${fixedProtocol}://${host}` : `${fixedProtocol}://${host}/${nfsExport}`;
            objectString =  (path) ? `${hostAndRootPath}/${path}` : hostAndRootPath ;
            break;
        }
        case 's3': {
            const {bucket} = details;
            let {prefix} = details;
            prefix = (prefix) ? (prefix.startsWith('/') ? prefix.substr(1) : prefix) : prefix;
            objectString = (prefix) ? `s3://${bucket}/${prefix}` : `s3://${bucket}`;
            break;
        }
        case 'azure': {
            const {container, storageAccountName} = details;
            let {prefix} = details;
            prefix = (prefix) ? (prefix.startsWith('/') ? prefix.substr(1) : prefix) : prefix;
            objectString = (prefix) ? `azure://${storageAccountName}/${container}/${prefix}` : `azure://${storageAccountName}/${container}`;
            break;
        }
        case 'gcp': {
            const {projectId, bucket} = details;
            let {prefix} = details;
            prefix = (prefix) ? (prefix.startsWith('/') ? prefix.substr(1) : prefix) : prefix;
            objectString = (prefix) ? `gcp://${projectId}/${bucket}/${prefix}` : `gcp://${projectId}/${bucket}`;
            break;
        }
        default: {
            objectString = "Unknown Protocol";
            break;
        }
    }
    return objectString;
}

export const defaultFilter = (item, filterString) => {
    if (_.isFunction(item)) return false;
    return _.values(item).find(value => {
        if (_.isEmpty(value) || !value) {
            return false;
        }
        else if(_.isObject(value)){
            return defaultFilter(value, filterString);
        }
        else if(_.isArray(value)){
            return false;
        }
        else return _.toString(value).toLowerCase().indexOf(filterString.toLowerCase()) !== -1;
    } );
};

function s4() {
    return Math.floor((1 + Math.random()) * 0x10000).toString(16).substring(1);
}

export const removeKeyFromPath = (path, key) => {
    if(_.startsWith(path, key))
        return _.replace(path, key, "");
    else return path;
};

export function uuid() {
    return s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4();
}

export const tagsToString = (tagsArray) => {
    const tagsString = _.reduce(tagsArray, (str, tag) => {
        return str ? (tag && tag.key && tag.value ? `${str},${tag.key}=${tag.value}` : str) : (tag && tag.key && tag.value ? `${tag.key}=${tag.value}` : "");
    }, "");
    return tagsString ? tagsString : "noTags";
};

export const stringToTags = (tagsString) => {
    const tagsArray = tagsString.split(",");
    return _.reduce(tagsArray, (tags, tagString) => {
        const splitTag = tagString.split("=");
        tags.push({key: splitTag[0], value: splitTag[1]});
        return tags;
    }, []);
};

export const selectText = (element) => {
    if (window.getSelection) {
        window.getSelection().removeAllRanges();
        let range = document.createRange();
        range.selectNode(element);
        window.getSelection().addRange(range);
    }
};

export const sendEvents = (gaAction, gaLabel, fbqEvent) => {
    window.ga && window.ga('send', {
        hitType: 'event',
        eventCategory: 'onboarding',
        eventAction: gaAction,
        eventLabel: gaLabel
    });

    window.fbq && window.fbq('track', fbqEvent);

};

export const getParameterByName = (name, url) => {
    if (!url) {
        url = window.location.href;
    }
    name = name.replace(/[[\]]/g, "\\$&");
    var regex = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)"),
        results = regex.exec(url);
    if (!results) return null;
    if (!results[2]) return '';
    return decodeURIComponent(results[2].replace(/\+/g, " "));
};
