import _ from 'lodash';
import React from 'react';


const SETTINGS_KEY = "settings";

export const SETTINGS_DEFAULTS = {
    gracePeriod: 30,
    deleteOnTarget: false,
    fileTypes: {
        files: true,
        directories: true,
        symlinks: true
    },
    retries: 3
};

export const fileTypesMap = {
    files: 'Files',
    directories: 'Directories',
    symlinks: 'Symbolic Links'
};

export const fileTypesOrder = [fileTypesMap.files, fileTypesMap.directories, fileTypesMap.symlinks];

export const NOT_EXCLUDE_FILES_OPTION = "Don’t exclude files that are modified before a scheduled sync";
export const EXCLUDE_FILES_OPTION = "Exclude files that are modified up to ";

export const NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT = "Never delete files from the target location";
export const DELETE_FILES_FROM_TARGET_OPTION_TEXT = "Delete files from the target location if they were deleted from the source";

export const saveSettingsToSessionStorage = (settings) => {
    window.sessionStorage.setItem(SETTINGS_KEY, JSON.stringify(settings));
};

export const getSettingsFromSessionStorage = () => {
    let settings = JSON.parse(window.sessionStorage.getItem(SETTINGS_KEY));
    if (settings === null) {
        settings = SETTINGS_DEFAULTS;
    }
    return settings;
};

export const removeSettingsFromSessionStorage = () => {
    window.sessionStorage.removeItem(SETTINGS_KEY);
};

export const fileTypesOptions = [
    {value: fileTypesMap.files, label: fileTypesMap.files},
    {value: fileTypesMap.directories, label: fileTypesMap.directories},
    {value: fileTypesMap.symlinks, label: fileTypesMap.symlinks}];

export const retriesOptions = [{label: "0", value: 0},{label: "1", value: 1},{label: "2", value: 2},{label: "3", value: 3},{label: "4", value: 4},{label: "5", value: 5}];

export const graceResolutionOptions = [{label: "Seconds", value: "Seconds"}, {label: "Minutes", value: "Minutes"}, {label: "Hours", value: "Hours"}];

export const getGraceSummary = (gracePeriodOn, graceValue, graceResolution) => {
    return _.includes(gracePeriodOn, NOT_EXCLUDE_FILES_OPTION) ?  NOT_EXCLUDE_FILES_OPTION :
        <div>{EXCLUDE_FILES_OPTION}<b> {graceValue} {_.get(graceResolution, 'label')}</b> before a scheduled sync</div>
};

export const getFileTypesSummary = (fileTypes) => {
    if (!fileTypes) return;
    fileTypes.sort((typeA, typeB) => {
        return (fileTypesOrder.indexOf(typeA) - fileTypesOrder.indexOf(typeB));
    });
    return fileTypes.join(', ');
};

export const getSummaryLines = (formValues) => {
    const includeAllTypes = _.get(formValues.fileTypes, 'length') === fileTypesOrder.length;
    return {
        gracePeriod: getGraceSummary(formValues.gracePeriodOn, formValues.graceValue, formValues.graceResolution),
        deleteOnTarget: formValues.deleteOnTarget,
        fileTypes: `Include ${includeAllTypes ? 'All' : 'Only'}: ${getFileTypesSummary(formValues.fileTypes)}`,
        retries: `Retry ${_.get(formValues.retries, 'value')} ${_.get(formValues.retries, 'value') === 1 ? 'time' : 'times'} before skipping file`
    };
};

export const getGracePeriod = (graceValue, graceResolution, isGraceEnabled) => {
    const res = graceResolution.value;
    const power = res==='Hours' ? 3600 : (res==='Minutes' ? 60 : 1);
    const grace = isGraceEnabled ? graceValue : 0;
    return grace*power;
};

export const getFormGraceResolution = (seconds) => {
    if (seconds < 60) return {count: seconds, resolution: {label: "Seconds", value: "Seconds"}};
    else if (seconds < 3600) return {count: (seconds / 60), resolution: {label: "Minutes", value: "Minutes"}};
    else return {count: (seconds / 3600), resolution: {label: "Hours", value: "Hours"}};
};

export const getDeleteOnTargetBool = (deleteOnTarget) => {
    return _.includes(deleteOnTarget, DELETE_FILES_FROM_TARGET_OPTION_TEXT);
};

export const getFormDeleteOnTarget = (isDeleteOnTarget) => {
    return isDeleteOnTarget ? DELETE_FILES_FROM_TARGET_OPTION_TEXT : NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT;
};

export const getFileTypesObj = (typesArray) => {
    return {
        files: _.includes(typesArray, fileTypesMap.files),
        directories: _.includes(typesArray, fileTypesMap.directories),
        symlinks: _.includes(typesArray, fileTypesMap.symlinks)
    }
};

export const getFormFileTypes = (fileTypesObj) => {
    const typesArray = [];
    _.forEach(fileTypesObj, (value, type) => {
        if (value) typesArray.push(fileTypesMap[type]);
    });
    return typesArray;
};

export const createSettingsObjectFromFormValues = (values) => {
    return {
        gracePeriod: getGracePeriod(values.graceValue, values.graceResolution, !_.includes(values.gracePeriodOn, NOT_EXCLUDE_FILES_OPTION)),
        deleteOnTarget: getDeleteOnTargetBool(values.deleteOnTarget),
        fileTypes: getFileTypesObj(values.fileTypes),
        retries: values.retries.value,
    };
};

export const getFormValuesFromSettings = (settings) => {
    return {
        graceValue: getFormGraceResolution(settings.gracePeriod).count,
        graceResolution: getFormGraceResolution(settings.gracePeriod).resolution,
        gracePeriodOn: settings.gracePeriod > 0 ? EXCLUDE_FILES_OPTION : NOT_EXCLUDE_FILES_OPTION,
        deleteOnTarget: getFormDeleteOnTarget(settings.deleteOnTarget),
        fileTypes: getFormFileTypes(settings.fileTypes),
        retries: {label: `${settings.retries}`, value: settings.retries}
    };
};