import {syncReducer, defaultState} from "./../../src/reducers/reducer"
import * as actions from "./../../src/action_creators_consts"
import * as consts from "./../../src/consts";
import {initTests} from "./../globals";
import {SETTINGS_DEFAULTS} from "./../../src/utils/settings-utils";

beforeEach(() => {
    initTests();
    window.Intercom = jest.fn();
});


const withAuth = (_config) => defaultState(_config, {}).set("auth", {data: {eamil: "blabla"}});


const stepNamesByWizardType = {
    "s3-nfs": [consts.FM_AGENT_NO_NetApp, "S3 Bucket", "NFS Server", "Directories", "Review"],
    "s3-cifs": [consts.FM_AGENT_NO_NetApp, "S3 Bucket", "CIFS Server", "Shares", "Review"],
    "nfs-s3": ["NFS Server", consts.FM_AGENT_NO_NetApp, "Directories", "S3 Bucket", "Relationship Tags", "Review"],
    "cifs-s3": ["CIFS Server", consts.FM_AGENT_NO_NetApp, "Shares", "S3 Bucket", "Review"],
    "nfs-sg": ["NFS Server", consts.FM_AGENT_NO_NetApp, "Directories", "SG Server", "SG Bucket", "Review"],
    "cifs-sg": ["CIFS Server", consts.FM_AGENT_NO_NetApp, "Shares", "SG Server", "SG Bucket", "Review"]
};

const stepNameToPathName = {
    "Data Broker": "agents",
    "S3 Bucket": "buckets",
    "SG Bucket": "sgbuckets",
    "NFS Server": "nfs",
    "CIFS Server": "cifs",
    "SG Server": "sg",
    "Directories": "exports",
    "Shares": "shares",
    "Relationship Tags": "tags",
    "Review": "review"
};


const callSyncReducer = (state, action, urlParams = {location: {pathname: ""}, params: {}}) => {
    const updatedState = state.set("_routing", urlParams);
    return syncReducer(updatedState, action);
};


describe("reducer", () => {
    test("should return the initial state on call with undefined and no matched action", () => {
        expect(syncReducer(undefined, {}))
            .toEqual(defaultState());
    });

    describe("Login", () => {
        test("should reset state to default, but keep _config and add loginToken to localStorage on success", () => {
            const state = syncReducer(defaultState({test: "test"}).set("something", true), {
                type: actions.LOGIN_LOCAL,
                status: actions.STATUS_SUCCESS,
                payload: {authorization: "abc"}
            });

            expect(state._config).toEqual({"test": "test"});
            expect(state.something).toEqual(undefined);
        });

        test("should boot intercom on success", () => {

            syncReducer(defaultState({
                test: "test",
                succeeded: true,
                data: {intercomAppId: "testId"}
            }).set("something", true), {
                type: actions.LOGIN_LOCAL,
                status: actions.STATUS_SUCCESS,
                payload: {authorization: "abc", email: "email@email.com"}
            });

            expect(window.Intercom.mock.calls[0][0]).toEqual("boot");

            expect(window.Intercom.mock.calls[0][1].app_id).toEqual("testId");
            expect(window.Intercom.mock.calls[0][1].email).toEqual("email@email.com");
        })
    });

    describe("Logout", () => {
        test("delete keys from localStorage, shutdown intercom, remove auth from state", () => {
            window.localStorage.loginToken = "something";

            const startState = syncReducer(defaultState({
                test: "test",
                succeeded: true,
                data: {intercomAppId: "testId"}
            }), {type: actions.LOGIN_LOCAL, status: actions.STATUS_SUCCESS, payload: {authorization: "abc"}});
            const state = syncReducer(startState, {type: actions.LOGOUT});

            expect(state.auth).toBeUndefined();
            // expect(window.localStorage.loginToken).toBeUndefined(); //auth doesn't clear the loginToken on logout
            expect(window.Intercom.mock.calls[1][0]).toEqual("shutdown");
        })
    });

    describe("UI STATE", () => {
        test("should push the payload on push", () => {
            const state = syncReducer(withAuth().set("_ui", {states: []}), {
                type: actions.PUSH_UI_STATE,
                payload: {test: "test"}
            });

            expect(state._ui.states).toEqual([{test: "test"}]);
        });

        test("should start with an empty list if there is nothing when being pushed", () => {
            const state = syncReducer(withAuth().set("_ui", {}), {
                type: actions.PUSH_UI_STATE,
                payload: {test: "test"}
            });

            expect(state._ui.states).toEqual([{test: "test"}]);
        });

        test("should pop on pop", () => {
            const state = syncReducer(withAuth().set("_ui", {states: [{test: "test"}]}), {type: actions.POP_UI_STATE});
            expect(state._ui.states).toEqual([])
        });
    });

    describe("Get Data Brokers", testUpdateKeyWithBuiltState("_dataBrokers", actions.GET_DATA_BROKERS, false, [{}], [{
        creationFrom: "A few seconds ago",
        isWaiting: true,
        isFailed: false,
        isReady: false
    }]));
    describe("Create Data Broker", testUpdateKeyWithBuiltState("_createDataBroker", actions.DATA_BROKER, false));
    describe("Delete Data Broker", testUpdateKeyWithBuiltState("_deleteDataBroker", actions.DELETE_DATA_BROKER));
    describe("Sync Now", testUpdateKeyWithBuiltState("_syncNow", actions.SYNC_NOW));

    describe("Show exports", () => {
        const host = '1.1.1.1';
        const dataBrokerId = 1;
        const extra = {host, dataBrokerId};

        const buildStateWithExportsFolders = () => withAuth()
            .set("_showExports", {
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }]/*,
                             retrievedAll: true*/
                        },
                    },
                    id: 1
                }
            });

        const buildStateWithExportsDeepFolders = () => withAuth()
            .set("_showExports", {
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }]/*,
                             retrievedAll: true*/
                        },
                        '/logs': {
                            data: [{
                                actualPath: '/logs/folder1',
                                path: '/folder1'
                            }]
                        }
                    },
                    id: 1
                }
            });

        test("should update inProgress", () => {
            const state = syncReducer(withAuth(), {
                type: actions.SHOW_EXPORTS,
                status: actions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: true,
                    succeeded: false,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should update failed", () => {
            const state = syncReducer(withAuth(), {
                type: actions.SHOW_EXPORTS,
                status: actions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: true,
                    inProgress: false,
                    succeeded: false,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should update success", () => {
            const state = syncReducer(withAuth(), {
                type: actions.SHOW_EXPORTS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should keep all the folders from the state ( host & dataBrokerId are identical)", () => {
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: actions.SHOW_EXPORTS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {data: [{actualPath: '/data1', host, path: '/data1'}]}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should keep all the folders from the state -deep folders check ( host & dataBrokerId are identical)", () => {
            const state = syncReducer(buildStateWithExportsDeepFolders(), {
                type: actions.SHOW_EXPORTS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {data: [{actualPath: '/data1', host, path: '/data1'}]},
                        '/logs': {
                            data: [{
                                actualPath: '/logs/folder1',
                                path: '/folder1'
                            }]
                        }
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should not keep all the folders from the state ( host is different)", () => {
            const host = "2.2.2.2";
            const extra = {dataBrokerId, host};

            const initialState = buildStateWithExportsFolders();
            const state = syncReducer(initialState, {
                type: actions.SHOW_EXPORTS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    "dataBrokerId": 1,
                    "failed": false,
                    "folders": {
                        "-": {
                            "data": [
                                {
                                    "actualPath": "/data1",
                                    "host": "1.1.1.1",
                                    "path": "/data1"
                                }
                            ]
                        }
                    },
                    "host": "1.1.1.1",
                    "id": 1,
                    "inProgress": false,
                    "succeeded": true
                },
                "2---252E2---252E2---252E2": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should not keep all the folders from the state -deep folders check ( host is different)", () => {
            const host = "2.2.2.2";
            const extra = {dataBrokerId, host};

            const state = syncReducer(buildStateWithExportsDeepFolders(), {
                type: actions.SHOW_EXPORTS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    "dataBrokerId": 1,
                    "failed": false,
                    "folders": {
                        "-": {
                            "data": [
                                {
                                    "actualPath": "/data1",
                                    "host": "1.1.1.1",
                                    "path": "/data1"
                                }
                            ]
                        },
                        "/logs": {
                            "data": [
                                {
                                    "actualPath": "/logs/folder1",
                                    "path": "/folder1"
                                }
                            ]
                        }
                    },
                    "host": "1.1.1.1",
                    "id": 1,
                    "inProgress": false,
                    "succeeded": true
                },
                "2---252E2---252E2---252E2": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should not keep all the folders from the state ( dataBrokerId is different)", () => {
            const dataBrokerId = 2;
            const extra = {dataBrokerId, host};
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: actions.SHOW_EXPORTS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should not keep all the folders from the state -deep folders check ( dataBrokerId is different)", () => {
            const dataBrokerId = 2;
            const extra = {dataBrokerId, host};
            const state = syncReducer(buildStateWithExportsDeepFolders(), {
                type: actions.SHOW_EXPORTS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });


    });

    describe("Show shares", () => {
        const username = 'testUser';
        const host = '1.1.1.1';
        const dataBrokerId = 1;
        const domain = 'netapp';
        const password = 'Netapp234';
        const extra = {host, dataBrokerId, username, password, domain};

        const buildStateWithSharesFolders = () => withAuth()
            .set("_showShares", {
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    domain,
                    host,
                    username,
                    password,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }]/*,
                             retrievedAll: true*/
                        }
                    },
                    id: 1
                }
            });

        test("should update inProgress", () => {
            const state = syncReducer(withAuth(), {
                type: actions.SHOW_SHARES,
                status: actions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: true,
                    succeeded: false,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should update failed", () => {
            const state = syncReducer(withAuth(), {
                type: actions.SHOW_SHARES,
                status: actions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: true,
                    inProgress: false,
                    succeeded: false,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should update success", () => {
            const state = syncReducer(withAuth(), {
                type: actions.SHOW_SHARES,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should keep all the folders from the state ( username, password and domain are identical)", () => {
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: actions.SHOW_SHARES,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {data: [{actualPath: '/data1', host, path: '/data1'}]}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });


        test("should delete all the folders from the state  (different password)", () => {
            const newPassword = 'Netapp456';
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: actions.SHOW_SHARES,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, username, password: newPassword, domain}
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    username,
                    password: newPassword,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should delete all the folders from the state  (different username)", () => {
            const username = 'newUser2';
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: actions.SHOW_SHARES,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, username, password, domain}
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should delete all the folders from the state  (different domain)", () => {
            const domain = 'domain2';
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: actions.SHOW_SHARES,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, username, password, domain}
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should delete all the folders from the state  (different host)", () => {
            const host = '2.2.2.2';
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: actions.SHOW_SHARES,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, username, password, domain}
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1": {
                    "dataBrokerId": 1,
                    "domain": "netapp",
                    "failed": false,
                    "folders": {
                        "-": {
                            "data": [
                                {
                                    "actualPath": "/data1",
                                    "host": "1.1.1.1",
                                    "path": "/data1"
                                }
                            ]
                        }
                    },
                    "host": "1.1.1.1",
                    "id": 1,
                    "inProgress": false,
                    "password": "Netapp234",
                    "succeeded": true,
                    "username": "testUser"
                },
                "2---252E2---252E2---252E2": {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should delete all the folders from the state  (different dataBrokerId)", () => {
            const dataBrokerId = 2;
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: actions.SHOW_SHARES,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, username, password, domain}
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });
    });

    describe("Update shares", () => {

        const username = 'testUser';
        const host = '1.1.1.1';
        const dataBrokerId = 1;
        const domain = 'netapp';
        const password = 'Netapp234';
        const extra = {host, dataBrokerId, username, password, domain};
        const path = 'data1';

        const buildStateWithSharesFolders = () => withAuth()
            .set("_showShares", {
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    domain,
                    host,
                    username,
                    password,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: 'data1'
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }]/*,
                             retrievedAll: true*/
                        }
                    },
                    id: 1
                }
            });

        const buildStateWithoutSharesFolders = () => withAuth()
            .set("_showShares", {
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    domain,
                    host,
                    username,
                    password,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {}
                    },
                    id: 1
                }
            });

        test("should update retrievedAll and failedRetrieve upon failure in list-cifs-share-folders", () => {
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: actions.UPDATE_SHARES, payload: {
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "FAILED",
                        host
                    }
                }, status: actions.STATUS_ERROR, id: 1, extra
            });
            expect(state._showShares).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        username,
                        password,
                        domain,
                        failed: false,
                        failedRetrieve: true,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: 'data1'
                        }],
                        folders: {
                            "-": {
                                data: [{
                                    actualPath: "/data1",
                                    host: "1.1.1.1",
                                    path: "/data1"
                                }],
                                retrievedAll: true
                            }
                        },
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });


        test("should update exports upon status done in list-cifs-shares and the state doesnt have any exports", () => {
            const state = syncReducer(buildStateWithoutSharesFolders(), {
                type: actions.UPDATE_SHARES, payload: {
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host,
                        paths: [path]
                    }
                }, status: actions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showShares).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        username,
                        password,
                        domain,
                        host,
                        failedRetrieve: false,
                        retrievedAll: true,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path
                                    }
                                ],
                                retrievedAll: true
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            }
                        ],
                        id: 1
                    }
                }
            );
        });

        test("should update exports upon status done in list-cifs-shares and the state has previous exports", () => {
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: actions.UPDATE_SHARES, payload: {
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host,
                        paths: ['data2']
                    }
                }, status: actions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showShares).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        username,
                        password,
                        domain,
                        host,
                        retrievedAll: true,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path: '/' + path
                                    },
                                    {
                                        actualPath: "/data2",
                                        host,
                                        path: "data2"
                                    }
                                ],
                                retrievedAll: true
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            },
                            {
                                actualPath: "/data2",
                                host,
                                path: "data2"
                            }
                        ],
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll and export upon status DONE in list-cifs-shares and the state doesnt have any exports", () => {
            const state = syncReducer(buildStateWithoutSharesFolders(), {
                type: actions.UPDATE_SHARES, payload: {
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host,
                        paths: []
                    }
                }, status: actions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showShares).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        data: [],
                        folders: {
                            "-": {
                                retrievedAll: true,
                                data: []
                            }
                        },
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll and export upon status DONE in list-cifs-shares and the state has previous exports", () => {
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: actions.UPDATE_SHARES, payload: {
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host,
                        paths: []
                    }
                }, status: actions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showShares).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        username,
                        password,
                        domain,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path: '/' + path
                                    }
                                ],
                                retrievedAll: true
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            }
                        ],
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });

        test("should return state with unhandledMessages - _showShares is undefined", () => {
            const action = {
                type: actions.UPDATE_SHARES, payload: {
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host: '2.2.2.2'
                    }, status: actions.STATUS_SUCCESS, id: 1, extra
                }
            };

            const state = syncReducer(withAuth(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(withAuth().set("unhandledMessages", [action]));
        });

        test("should return state with unhandledMessages - host is not like _showShares.host", () => {

            const action = {
                type: actions.UPDATE_SHARES, payload: {
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host: '2.2.2.2'
                    }, status: actions.STATUS_SUCCESS, id: 1, extra
                }
            };

            const state = syncReducer(buildStateWithoutSharesFolders(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(buildStateWithoutSharesFolders().set("unhandledMessages", [action]));

        });

    });

    describe("list folders", () => {
        const dataBrokerId = 1;
        const fileServer = '1.1.1.1';
        const path = '/data1';
        const extra = {fileServer, path};

        const buildStateWithExportsFolders = () => withAuth()
            .set("_showExports", {
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host: fileServer,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host: fileServer,
                        path: '/data1'
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host: fileServer,
                                path: '/data1'
                            }],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });

        test("should update inProgress", () => {
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: actions.LIST_FOLDERS,
                status: actions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host: fileServer,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host: fileServer,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host: fileServer, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: false,
                            id: 1,
                            inProgress: true,
                            requestId: undefined,
                            succeeded: false
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            })
        });

        test("should update failed", () => {
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: actions.LIST_FOLDERS,
                status: actions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host: fileServer,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host: fileServer,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host: fileServer, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: true,
                            id: 1,
                            inProgress: false,
                            requestId: undefined,
                            succeeded: false
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });

        });

        test("should update success", () => {

            const state = syncReducer(buildStateWithExportsFolders(), {
                type: actions.LIST_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host: fileServer,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host: fileServer,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host: fileServer, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: false,
                            id: 1,
                            inProgress: false,
                            requestId: undefined,
                            succeeded: true
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });
        });

        test("should return undefined", () => {

            const state = syncReducer(withAuth(), {
                type: actions.LIST_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual(undefined);
            expect(state).toEqual(withAuth());
        });

        test("should return undefined and the same state that was sent- state._showExports.data is not exist", () => {

            const state = syncReducer(withAuth(), {
                type: actions.LIST_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual(undefined);
            expect(state).toEqual(withAuth());
        });

        test("Should return the same state that was sent - state._showExports.data = []", () => {
            const prevState = withAuth()
                .set("_showExports", {
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host: fileServer,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [],
                        folders: {
                            "-": {
                                data: [],
                                retrievedAll: true
                            }
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState, {
                type: actions.LIST_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent not state._showExports.data", () => {
            const prevState = withAuth()
                .set("_showExports", {
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host: fileServer,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            '-': {}
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState, {
                type: actions.LIST_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

    });


    describe("list buckets folders", () => {
        const dataBrokerId = 1;
        const bucket = 'thisBucket';
        const extra = {path: bucket, bucket, dataBrokerId};

        const buildStateWithBucketsFolders = () => withAuth()
            .set("_showBuckets",
                {
                    "1": {
                        dataBrokerId,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: bucket,
                                    name: bucket,
                                    path: bucket
                                }],
                                retrievedAll: true
                            }
                        },
                        retrievedAll: true,
                        id: 1
                    }
                });

        test("should update in progress for data1", () => {
            const state = syncReducer(buildStateWithBucketsFolders(), {
                type: actions.LIST_BUCKETS_FOLDERS,
                status: actions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showBuckets["1"]).toEqual({
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    'thisBucket': {
                        data: [],
                        failed: false,
                        inProgress: true,
                        succeeded: false,
                        error: undefined,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                id: 1
            })
        });

        test("should update failed", () => {
            const state = syncReducer(buildStateWithBucketsFolders(), {
                type: actions.LIST_BUCKETS_FOLDERS,
                status: actions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showBuckets).toEqual({
                "1" : {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: bucket,
                                name: bucket,
                                path: bucket
                            }],
                            retrievedAll: true
                        },
                        "thisBucket": {
                            data: [],
                            failed: true,
                            inProgress: false,
                            succeeded: false,
                            error: undefined,
                            requestId: undefined,
                            id: 1
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
                }
                );

        });

        test("should update success", () => {

            const state = syncReducer(buildStateWithBucketsFolders(), {
                type: actions.LIST_BUCKETS_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showBuckets).toEqual({
                "1" : {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: bucket,
                                name: bucket,
                                path: bucket
                            }],
                            retrievedAll: true
                        },
                        "thisBucket": {
                            data: [],
                            failed: false,
                            inProgress: false,
                            succeeded: true,
                            error: undefined,
                            requestId: undefined,
                            id: 1
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
                }
                );
        });

        test("should return undefined", () => {

            const state = syncReducer(withAuth(), {
                type: actions.LIST_BUCKETS_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showBuckets).toEqual(undefined);
            expect(state).toEqual(withAuth());
        });

        test("should return undefined and the same state that was sent- state._showBuckets.data is not exist", () => {
            const show =
                {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                };

            const state = syncReducer(withAuth()
                    .set("_showBuckets", show)
                , {
                    type: actions.LIST_BUCKETS_FOLDERS,
                    status: actions.STATUS_SUCCESS,
                    id: 1,
                    extra
                });

            expect(state._showBuckets).toEqual(show);
            expect(state).toEqual(withAuth().set("_showBuckets", show));
        });

        test("Should return the same state that was sent - state._showBuckets.data = []", () => {
            const prevState = withAuth()
                .set("_showBuckets", {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                });
            const currState = syncReducer(prevState, {
                type: actions.LIST_BUCKETS_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent not state._showBuckets.data", () => {
            const prevState = withAuth()
                .set("_showBuckets", {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {}
                    },
                    id: 1
                });
            const currState = syncReducer(prevState, {
                type: actions.LIST_BUCKETS_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent - bucket name does not exist in data", () => {
            const extra = {bucket: 'test', path: bucket};

            const state = syncReducer(buildStateWithBucketsFolders(), {
                type: actions.LIST_BUCKETS_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showBuckets).toEqual({
                "1" : {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: bucket,
                                name: bucket,
                                path: bucket
                            }],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
                }
                );

        });
    });


    describe("list SG buckets folders", () => {
        const dataBrokerId = 1;
        const bucket = 'thisBucket';
        const host = '1.1.1.1';
        const path = 'data1';
        const extra = {path, bucket, host};
        const port = 8080;
        const accessKey = 'a';
        const secretKey = 'a';

        const buildStateWithSGBucketsFolders = () => withAuth()
            .set("_showSGBuckets", {'1.1.1.1' : {
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    }
                },
                accessKey,
                secretKey,
                port,
                retrievedAll: true,
                id: 1
            }});

        test("should update in progress for data1", () => {
            const state = syncReducer(buildStateWithSGBucketsFolders(), {
                type: actions.LIST_SG_BUCKETS_FOLDERS,
                status: actions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showSGBuckets).toEqual({ '1.1.1.1': {
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    "data1": {
                        data: [],
                        failed: false,
                        inProgress: true,
                        succeeded: false,
                        error: undefined,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                accessKey,
                secretKey,
                port,
                id: 1
            }})
        });

        test("should update failed", () => {
            const state = syncReducer(buildStateWithSGBucketsFolders(), {
                type: actions.LIST_SG_BUCKETS_FOLDERS,
                status: actions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showSGBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    "data1": {
                        data: [],
                        failed: true,
                        inProgress: false,
                        succeeded: false,
                        error: undefined,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                accessKey,
                secretKey,
                port,
                id: 1
            }});

        });

        test("should update success", () => {

            const state = syncReducer(buildStateWithSGBucketsFolders(), {
                type: actions.LIST_SG_BUCKETS_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showSGBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    "data1": {
                        data: [],
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        error: undefined,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                accessKey,
                secretKey,
                port,
                id: 1
            }});
        });

        test("should return undefined", () => {

            const state = syncReducer(withAuth(), {
                type: actions.LIST_SG_BUCKETS_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showBuckets).toEqual(undefined);
            expect(state).toEqual(withAuth());
        });

        test("should return undefined and the same state that was sent- state._showSGBuckets.data is not exist", () => {

            const state = syncReducer(withAuth(), {
                type: actions.LIST_SG_BUCKETS_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showSGBuckets).toEqual(undefined);
            expect(state).toEqual(withAuth());
        });

        test("Should return the same state that was sent - state._showSGBuckets.data = []", () => {
            const prevState = withAuth()
                .set("_showSGBuckets", {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    accessKey,
                    secretKey,
                    port,
                    id: 1
                });
            const currState = syncReducer(prevState, {
                type: actions.LIST_SG_BUCKETS_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent not state._showSGBuckets.data", () => {
            const prevState = withAuth()
                .set("_showSGBuckets", {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {}
                    },
                    accessKey,
                    secretKey,
                    port,
                    id: 1
                });
            const currState = syncReducer(prevState, {
                type: actions.LIST_SG_BUCKETS_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent bucket name does not exist in data", () => {
            const extra = {bucket: 'test', path};

            const state = syncReducer(buildStateWithSGBucketsFolders(), {
                type: actions.LIST_SG_BUCKETS_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showSGBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    }
                },
                accessKey,
                secretKey,
                port,
                retrievedAll: true,
                id: 1
            }});

        });
    });


    describe('update folders', () => {
        const dataBrokerId = 1;
        const host = '1.1.1.1';

        const buildStateWithExportsFoldersAndAfterListFolders = () => withAuth()
            .set("_showExports", {
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            retrievedAll: true
                        },
                        '/data1': {data: [], failed: false, inProgress: false, succeeded: true}
                    },
                    id: 1
                }
            });


        test("should update retrievedAll  upon failure in list-folders", () => {
            const action = {
                type: actions.UPDATE_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "FAILED",
                        path: '/data1',
                        nfsServer: host
                    }
                }, status: actions.STATUS_ERROR, id: 1
            };

            const state = syncReducer(buildStateWithExportsFoldersAndAfterListFolders(), action);

            expect(state._showExports).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {data: [], failed: false, inProgress: false, succeeded: true, retrievedAll: true}
                        },
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll after success in list-folders", () => {

            const action = {
                type: actions.UPDATE_FOLDERS,
                payload: {
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        nfsServer: host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: actions.STATUS_SUCCESS, id: 1
            };

            const state = syncReducer(buildStateWithExportsFoldersAndAfterListFolders(), action);

            expect(state._showExports).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [
                                    {
                                        actualPath: "/data1/a",
                                        path: "a"
                                    },
                                    {
                                        actualPath: "/data1/ab",
                                        path: "ab"
                                    },
                                    {
                                        actualPath: "/data1/ac",
                                        path: "ac"
                                    }
                                ], failed: false, inProgress: false, succeeded: true, retrievedAll: true
                            }
                        },
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll after success in list-folders and to unite between the folders ( the ones from the action and ones from the state ) ", () => {
            const action = {
                type: actions.UPDATE_FOLDERS,
                payload: {
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        nfsServer: host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: actions.STATUS_SUCCESS, id: 1
            };

            const state = syncReducer(withAuth()
                    .set("_showExports", {
                        "1---252E1---252E1---252E1": {
                            dataBrokerId,
                            host,
                            failed: false,
                            inProgress: false,
                            succeeded: true,
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            folders: {
                                '-': {
                                    data: [{
                                        actualPath: '/data1',
                                        host,
                                        path: '/data1'
                                    }],
                                    retrievedAll: true
                                },
                                '/data1': {
                                    data: [{
                                        actualPath: "/data1/b",
                                        path: "b"
                                    },
                                        {
                                            actualPath: "/data1/c",
                                            path: "c"
                                        },
                                        {
                                            actualPath: "/data1/d",
                                            path: "d"
                                        }], failed: false, inProgress: false, succeeded: true
                                }
                            },
                            id: 1
                        }
                    })
                , action);

            expect(state._showExports).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [
                                    {
                                        actualPath: "/data1/b",
                                        path: "b"
                                    },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    },
                                    {
                                        actualPath: "/data1/a",
                                        path: "a"
                                    },
                                    {
                                        actualPath: "/data1/ab",
                                        path: "ab"
                                    },
                                    {
                                        actualPath: "/data1/ac",
                                        path: "ac"
                                    }
                                ], failed: false, inProgress: false, succeeded: true, retrievedAll: true
                            }
                        },
                        id: 1
                    }
                }
            );
        });

        test("should return undefined and the same state that was sent- state._showExports is not exist", () => {

            const action = {
                type: actions.UPDATE_FOLDERS,
                payload: {
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        nfsServer: host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: actions.STATUS_SUCCESS, id: 1
            };

            const state = syncReducer(withAuth(), action);

            expect(state._showExports).toEqual(undefined);
            expect(state).toEqual(withAuth());

        });

        test("should return undefined and the same state that was sent- state._showExports.data is not exist", () => {
            const prevState = withAuth()
                .set("_showExports", {
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [{
                                    actualPath: "/data1/b",
                                    path: "b"
                                },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    }], failed: false, inProgress: false, succeeded: true
                            }
                        },
                        id: 1
                    }
                });

            const currState = syncReducer(prevState, {
                type: actions.UPDATE_FOLDERS,
                payload: {
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        nfsServer: host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: actions.STATUS_SUCCESS, id: 1
            });

            expect(prevState).toEqual(currState);

        });

        test("should return undefined and the same state that was sent- state._showExports.data=[] ", () => {
            const prevState = withAuth()
                .set("_showExports", {
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [{
                                    actualPath: "/data1/b",
                                    path: "b"
                                },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    }], failed: false, inProgress: false, succeeded: true
                            }
                        },
                        id: 1
                    }
                });

            const currState = syncReducer(prevState, {
                type: actions.UPDATE_FOLDERS,
                payload: {
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        nfsServer: host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: actions.STATUS_SUCCESS, id: 1
            });

            expect(currState).toEqual(prevState);

        });

        test("should return undefined and the same state that was sent- state._showExports.data=[] ", () => {
            const prevState = withAuth()
                .set("_showExports", {
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [{
                                    actualPath: "/data1/b",
                                    path: "b"
                                },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    }], failed: false, inProgress: false, succeeded: true
                            }
                        },
                        id: 1
                    }
                });

            const currState = syncReducer(prevState, {
                type: actions.UPDATE_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        nfsServer: host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: actions.STATUS_SUCCESS, id: 1
            });

            expect(currState).toEqual(prevState);

        });

        test("Should return the same state that was sent _showExports.folders[path] in undefined", () => {
            const prevState = withAuth()
                .set("_showExports", {
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host: '2.2.2.2',
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            }
                        },
                        id: 1
                    }
                });

            const currState = syncReducer(prevState, {
                type: actions.UPDATE_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        nfsServer: host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: actions.STATUS_SUCCESS, id: 1
            });

            expect(currState).toEqual(prevState);

        });

    });

    describe("list shares folders", () => {
        const dataBrokerId = 1;
        const host = '1.1.1.1';
        const path = '/data1';
        const extra = {host, path};


        const buildStateWithExportsFolders = () => withAuth()
            .set("_showShares", {
                "1---252E1---252E1---252E1":  {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });

        test("should update inProgress", () => {
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: actions.LIST_SHARE_FOLDERS,
                status: actions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1":{
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: false,
                            id: 1,
                            inProgress: true,
                            requestId: undefined,
                            succeeded: false
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            })
        });

        test("should update failed", () => {
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: actions.LIST_SHARE_FOLDERS,
                status: actions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1":{
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: true,
                            id: 1,
                            inProgress: false,
                            requestId: undefined,
                            succeeded: false
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });

        });

        test("should update success", () => {

            const state = syncReducer(buildStateWithExportsFolders(), {
                type: actions.LIST_SHARE_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: false,
                            id: 1,
                            inProgress: false,
                            requestId: undefined,
                            succeeded: true
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });
        });

        test("should return undefined", () => {

            const state = syncReducer(withAuth(), {
                type: actions.LIST_SHARE_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual(undefined);
            expect(state).toEqual(withAuth());
        });

        test("should return undefined and the same state that was sent- state._showShares.data is not exist", () => {

            const state = syncReducer(withAuth(), {
                type: actions.LIST_SHARE_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual(undefined);
            expect(state).toEqual(withAuth());
        });

        test("Should return the same state that was sent - state._showShares.data = []", () => {
            const prevState = withAuth()
                .set("_showShares", {
                    "1---252E1---252E1---252E1":  {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [],
                        folders: {
                            "-": {
                                data: [],
                                retrievedAll: true
                            }
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState, {
                type: actions.LIST_SHARE_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent not state._showShares.data", () => {
            const prevState = withAuth()
                .set("_showShares", {
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            '-': {}
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState, {
                type: actions.LIST_SHARE_FOLDERS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

    });

    describe("update shares folders", () => {
        const username = 'testUser';
        const host = '1.1.1.1';
        const dataBrokerId = 1;
        const domain = 'netapp';
        const password = 'Netapp234';

        const buildStateWithSharesFoldersAndAfterListFolders = () => withAuth()
            .set("_showShares", {
                "1---252E1---252E1---252E1":{
                    dataBrokerId,
                    domain,
                    host,
                    username,
                    password,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            retrievedAll: true
                        },
                        '/data1': {data: [], failed: false, inProgress: false, succeeded: true}
                    },
                    id: 1
                }
            });


        test("should update retrievedAll  upon failure in list-cifs-share-folders", () => {
            const state = syncReducer(buildStateWithSharesFoldersAndAfterListFolders(), {
                type: actions.UPDATE_SHARE_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                        status: "FAILED",
                        path: '',
                        share: '/data1',
                        host
                    }
                }, status: actions.STATUS_ERROR, id: 1
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1":  {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {data: [], failed: false, inProgress: false, succeeded: true, retrievedAll: true}
                        },
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll after success in list-cifs-share-folders", () => {
            const state = syncReducer(buildStateWithSharesFoldersAndAfterListFolders(), {
                type: actions.UPDATE_SHARE_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                        status: "DONE",
                        path: '',
                        share: '/data1',
                        host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: actions.STATUS_SUCCESS, id: 1
            });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [
                                    {
                                        actualPath: "/data1/a",
                                        path: "a"
                                    },
                                    {
                                        actualPath: "/data1/ab",
                                        path: "ab"
                                    },
                                    {
                                        actualPath: "/data1/ac",
                                        path: "ac"
                                    }
                                ], failed: false, inProgress: false, succeeded: true, retrievedAll: true
                            }
                        },
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll after success in list-cifs-share-folders and to unite between the folders ( the ones from the action and ones from the state ) ", () => {
            const state = syncReducer(withAuth()
                    .set("_showShares", {
                        "1---252E1---252E1---252E1":   {
                            dataBrokerId,
                            domain,
                            host,
                            username,
                            password,
                            failed: false,
                            inProgress: false,
                            succeeded: true,
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            folders: {
                                '-': {
                                    data: [{
                                        actualPath: '/data1',
                                        host,
                                        path: '/data1'
                                    }],
                                    retrievedAll: true
                                },
                                '/data1': {
                                    data: [{
                                        actualPath: "/data1/b",
                                        path: "b"
                                    },
                                        {
                                            actualPath: "/data1/c",
                                            path: "c"
                                        },
                                        {
                                            actualPath: "/data1/d",
                                            path: "d"
                                        }], failed: false, inProgress: false, succeeded: true
                                }
                            },
                            id: 1
                        }
                    })
                , {
                    type: actions.UPDATE_SHARE_FOLDERS, payload: {
                        [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                            status: "DONE",
                            path: '',
                            share: '/data1',
                            host,
                            folders: ['a', 'ab', 'ac']
                        }
                    }, status: actions.STATUS_SUCCESS, id: 1
                });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1":  {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [
                                    {
                                        actualPath: "/data1/b",
                                        path: "b"
                                    },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    },
                                    {
                                        actualPath: "/data1/a",
                                        path: "a"
                                    },
                                    {
                                        actualPath: "/data1/ab",
                                        path: "ab"
                                    },
                                    {
                                        actualPath: "/data1/ac",
                                        path: "ac"
                                    }
                                ], failed: false, inProgress: false, succeeded: true, retrievedAll: true
                            }
                        },
                        id: 1
                    }
                }
            );
        });

        test("should return the full path as required  ", () => {
            const state = syncReducer(withAuth()
                    .set("_showShares", {
                        "1---252E1---252E1---252E1":{
                            dataBrokerId,
                            domain,
                            host,
                            username,
                            password,
                            failed: false,
                            inProgress: false,
                            succeeded: true,
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            folders: {
                                '-': {
                                    data: [{
                                        actualPath: '/data1',
                                        host,
                                        path: '/data1'
                                    }],
                                    retrievedAll: true
                                },
                                '/data1': {
                                    data: [{
                                        actualPath: "/data1/b",
                                        path: "b"
                                    }], failed: false, inProgress: false, succeeded: true
                                },
                                '/data1/b': {data: [], failed: false, inProgress: false, succeeded: true}
                            },
                            id: 1
                        }
                    })
                , {
                    type: actions.UPDATE_SHARE_FOLDERS, payload: {
                        [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                            status: "DONE",
                            path: 'b',
                            share: '/data1',
                            host,
                            folders: ['a', 'ab', 'ac']
                        }
                    }, status: actions.STATUS_SUCCESS, id: 1
                });

            expect(state._showShares).toEqual({
                "1---252E1---252E1---252E1":{
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [
                                    {
                                        actualPath: "/data1/b",
                                        path: "b"
                                    }], failed: false, inProgress: false, succeeded: true
                            },
                            '/data1/b': {
                                data: [
                                    {
                                        actualPath: "/data1/b/a",
                                        path: "a"
                                    },
                                    {
                                        actualPath: "/data1/b/ab",
                                        path: "ab"
                                    },
                                    {
                                        actualPath: "/data1/b/ac",
                                        path: "ac"
                                    }], failed: false, inProgress: false, retrievedAll: true, succeeded: true
                            }
                        },
                        id: 1
                    }
                }
            );
        });


        test("should return undefined and the same state that was sent- state._showShares is not exist", () => {
            const state = syncReducer(withAuth()
                , {
                    type: actions.UPDATE_SHARE_FOLDERS, payload: {
                        [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                            status: "DONE",
                            path: '',
                            share: '/data1',
                            host,
                            folders: ['a', 'ab', 'ac']
                        }
                    }, status: actions.STATUS_SUCCESS, id: 1
                });

            expect(state._showShares).toEqual(undefined);
            expect(state).toEqual(withAuth());

        });

        test("should return undefined and the same state that was sent- state._showShares.data is not exist", () => {
            const prevState = withAuth()
                .set("_showShares", {
                    "1---252E1---252E1---252E1":{
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [{
                                    actualPath: "/data1/b",
                                    path: "b"
                                },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    }], failed: false, inProgress: false, succeeded: true
                            }
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState
                , {
                    type: actions.UPDATE_SHARE_FOLDERS, payload: {
                        [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                            status: "DONE",
                            path: '',
                            share: '/data1',
                            host,
                            folders: ['a', 'ab', 'ac']
                        }
                    }, status: actions.STATUS_SUCCESS, id: 1
                });

            expect(prevState).toEqual(currState);

        });

        test("should return undefined and the same state that was sent- state._showShares.data=[] ", () => {
            const prevState = withAuth()
                .set("_showShares", {
                    "1---252E1---252E1---252E1":{
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [{
                                    actualPath: "/data1/b",
                                    path: "b"
                                },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    }], failed: false, inProgress: false, succeeded: true
                            }
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState, {
                type: actions.UPDATE_SHARE_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                        status: "DONE",
                        path: '',
                        share: '/data1',
                        host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: actions.STATUS_SUCCESS, id: 1
            });

            expect(currState).toEqual(prevState);

        });

        test("should return undefined and the same state that was sent- state._showShares.data=[] ", () => {
            const prevState = withAuth()
                .set("_showShares", {
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [{
                                    actualPath: "/data1/b",
                                    path: "b"
                                },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    }], failed: false, inProgress: false, succeeded: true
                            }
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState, {
                type: actions.UPDATE_SHARE_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                        status: "DONE",
                        path: '',
                        share: '/data1',
                        host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: actions.STATUS_SUCCESS, id: 1
            });

            expect(currState).toEqual(prevState);

        });

        test("Should return the same state that was sent _showShares.folders[path] in undefined", () => {
            const prevState = withAuth()
                .set("_showShares", {
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host: '2.2.2.2',
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            }
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState
                , {
                    type: actions.UPDATE_SHARE_FOLDERS, payload: {
                        [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                            status: "DONE",
                            path: '',
                            share: '/data1',
                            host,
                            folders: ['a', 'ab', 'ac']
                        }
                    }, status: actions.STATUS_SUCCESS, id: 1
                });

            expect(currState).toEqual(prevState);

        });

    });

    describe("Show sg buckets", () => {
        const accessKey = '1';
        const secretKey = '1';
        const host = '1.1.1.1';
        const dataBrokerId = 1;
        const port = 1;
        const extra = {host, dataBrokerId, accessKey, secretKey, port};

        const buildStateWithSGBuckets = () => withAuth().set("_showSGBuckets", { '1.1.1.1' : {
            dataBrokerId,
            port,
            host,
            accessKey,
            secretKey,
            failed: false,
            inProgress: false,
            succeeded: true,
            folders: {'-': {data: [{actualPath: 'small', name: 'small', path: 'small'}], retrievedAll: true}},
            id: 1
        }});

        test("should update inProgress", () => {
            const state = syncReducer(withAuth(), {
                type: actions.SHOW_SG_BUCKETS,
                status: actions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showSGBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                host,
                accessKey,
                secretKey,
                port,
                failed: false,
                inProgress: true,
                succeeded: false,
                folders: {
                    "-": {}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            }})
        });

        test("should update failed", () => {
            const state = syncReducer(withAuth(), {
                type: actions.SHOW_SG_BUCKETS,
                status: actions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showSGBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                host,
                accessKey,
                secretKey,
                port,
                failed: true,
                inProgress: false,
                succeeded: false,
                folders: {
                    "-": {}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            }})
        });

        test("should update success", () => {
            const state = syncReducer(withAuth(), {
                type: actions.SHOW_SG_BUCKETS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showSGBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                host,
                accessKey,
                secretKey,
                port,
                failed: false,
                inProgress: false,
                succeeded: true,
                folders: {
                    "-": {}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            }})
        });

        test("should keep all the folders from the state ( port, awsAccessKey and awsSecretKey are identical)", () => {
            const state = syncReducer(buildStateWithSGBuckets(), {
                type: actions.SHOW_SG_BUCKETS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showSGBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                host,
                port,
                secretKey,
                accessKey,
                failed: false,
                inProgress: false,
                succeeded: true,
                folders: {
                    "-": {data: [{actualPath: 'small', name: 'small', path: 'small'}], retrievedAll: true}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            }})
        });


        test("should delete all the folders from the state  (different accessKey and secretKey)", () => {
            const accessKey = '2';
            const secretKey = '2';
            const state = syncReducer(buildStateWithSGBuckets(), {
                type: actions.SHOW_SG_BUCKETS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, accessKey, secretKey, port}
            });

            expect(state._showSGBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                host,
                accessKey,
                secretKey,
                port,
                failed: false,
                inProgress: false,
                succeeded: true,
                folders: {
                    "-": {}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            }})
        });

        test("should delete all the folders from the state  (different port)", () => {
            const port = 2;
            const state = syncReducer(buildStateWithSGBuckets(), {
                type: actions.SHOW_SG_BUCKETS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, accessKey, secretKey, port}
            });

            expect(state._showSGBuckets).toEqual({ '1.1.1.1' : {
                    dataBrokerId,
                    host,
                    accessKey,
                    secretKey,
                    port,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }})
        });

        test("should delete all the folders from the state  (different host)", () => {
            const host = "2.2.2.2";
            const state = syncReducer(buildStateWithSGBuckets(), {
                type: actions.SHOW_SG_BUCKETS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, accessKey, secretKey, port}
            });

            expect(state._showSGBuckets[host]).toEqual({
                dataBrokerId,
                host,
                accessKey,
                secretKey,
                port,
                failed: false,
                inProgress: false,
                succeeded: true,
                folders: {
                    "-": {}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            })
        });

        test("should delete all the folders from the state  (different dataBrokerId)", () => {
            const dataBrokerId = 2;
            const state = syncReducer(buildStateWithSGBuckets(), {
                type: actions.SHOW_SG_BUCKETS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, accessKey, secretKey, port}
            });

            expect(state._showSGBuckets).toEqual({ '1.1.1.1' : {
                    dataBrokerId,
                    host,
                    accessKey,
                    secretKey,
                    port,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }})
        });

    });

    describe("Show buckets", () => {
        const extra = {dataBrokerId : 1};
        test("should update inProgress", () => {
            const state = syncReducer(withAuth(), {type: actions.SHOW_BUCKETS, status: actions.STATUS_PROGRESS, id: 1, extra});

            expect(state._showBuckets).toEqual({
                "1": {
                    "failed": false,
                    "folders": {
                    "-": {}
                    },
                    "id": 1,
                    "inProgress": true,
                    "succeeded": false
                }
            })
        });

        test("should update failed", () => {
            const state = syncReducer(withAuth(), {type: actions.SHOW_BUCKETS, status: actions.STATUS_ERROR, id: 1, extra});

            expect(state._showBuckets).toEqual({
                "1": {
                failed: true,
                inProgress: false,
                error: undefined,
                requestId: undefined,
                succeeded: false,
                id: 1,
                folders: {
                    "-": {}
                }
                }
            })
        });

        test("should update success no showBuckets field", () => {
            const state = syncReducer(withAuth(), {type: actions.SHOW_BUCKETS, status: actions.STATUS_SUCCESS, id: 1, extra});

            expect(state._showBuckets).toEqual({
                "1": {
                failed: false,
                inProgress: false,
                error: undefined,
                requestId: undefined,
                succeeded: true,
                id: 1,
                folders: {
                    "-": {}
                }
                }
            })
        });

        test("should update success with showBuckets field", () => {
            const state = syncReducer(withAuth()
                .setIn(["_showBuckets","1"], {
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {
                            data: [{
                                actualPath: 'large',
                                name: 'large',
                                path: 'large',
                                region: {displayName: "EU (Frankfurt)", name: "eu-central-1"}, tags: []
                            }],
                            retrievedAll: true
                        }
                    },
                    id: 1
                }), {type: actions.SHOW_BUCKETS, status: actions.STATUS_SUCCESS, id: 1, extra});

            expect(state._showBuckets).toEqual({
                "1": {
                failed: false,
                inProgress: false,
                error: undefined,
                requestId: undefined,
                succeeded: true,
                id: 1,
                folders: {
                    '-': {
                        data: [{
                            actualPath: 'large',
                            name: 'large',
                            path: 'large',
                            region: {displayName: "EU (Frankfurt)", name: "eu-central-1"}, tags: []
                        }],
                        retrievedAll: true
                    }
                }
                }
            })
        });
    });


    describe("Get relationships", () => {

        const nfsToS3RelationshipResponse = {
            activity: {
                type: "Initial Copy",
                status: "RUNNING",
                capacityCopiedBytes: 900,
                executionTime: 72000000,
                lastMessageTime: undefined,
                progress: 94
            },
            source: {protocol: "nfs", "nfs" : {host: "1.1.1.1", export: "/data1"}},
            target: {protocol: "s3" , "s3": {bucket: "small"}},
            host: "nfsServer",
            sync: {
                autoSync: "ON",
                inMinutes: 5,
                inHours: 0,
                inDays: 0
            },
            actualTarget: "s3://small/1.1.1.1:@_data1",
            dataBroker: {transferRate: 350000000},
            group: {dataBrokers: [
                    {id: '123', placement: {awsAccountId: '123'}, transferRate: 350000000},
                    {id: '456'}
                ]}

        };

        const nfsToS3RelationshipResponseProcessed = Object.assign({}, nfsToS3RelationshipResponse, {
            group: {dataBrokers: [
                    {id: '123', placement: {awsAccountId: '123'}, creationFrom: "A few seconds ago", isDisabled: false, isFailed: false, isReady: true, isWaiting: false, transferRate: 350000000},
                    {id: '456', creationFrom: "A few seconds ago", isDisabled: undefined, isFailed: false, isReady: false, isWaiting: true},
                ]}
        });

        const nfsToS3RelationshipResult = Object.assign({}, nfsToS3RelationshipResponseProcessed, {
            ui: {
                actualTarget: 's3://small/1.1.1.1:@_data1',
                autoSync: 'ON',
                dataBrokers: [
                    {id: '123', placement: {awsAccountId: '123'}, creationFrom: "A few seconds ago", isDisabled: false, isFailed: false, isReady: true, isWaiting: false, transferRate: 350000000,
                        ui: {transferRate: "(333.79MB/s)", type: {name: 'AWS', displayName: 'AWS Data Broker', disabledIcon: '', colorIcon: '', whiteIcon: ''}}},
                    {id: '456', creationFrom: "A few seconds ago", isDisabled: undefined, isFailed: false, isReady: false, isWaiting: true,
                        ui: {transferRate: "", type: {name: 'AWS', displayName: 'AWS Data Broker', disabledIcon: '', colorIcon: '', whiteIcon: ''}}},
                ],
                gracePeriod: SETTINGS_DEFAULTS.gracePeriod,
                deletionOn: SETTINGS_DEFAULTS.deleteOnTarget,
                formattedBytesCopied: {
                    size: '0',
                    suffix: 'B'
                },
                formattedBytesFailed: {
                    size: '0',
                    suffix: 'B'
                },
                formattedBytesMarkedForCopy: {
                    size: '0',
                    suffix: 'B'
                },
                formattedBytesMarkedForRemove: {
                    size: '0',
                    suffix: 'B'
                },
                formattedBytesRemoved: null,
                formattedBytesScanned: {
                    size: '0',
                    suffix: 'B'
                },
                formattedDirsFailedToScan: '0',
                formattedDirsMarkedForCopy: '0',
                formattedDirsMarkedForRemove: '0',
                formattedDirsScanned: '0',
                formattedFilesCopied: '0',
                formattedFilesFailed: '0',
                formattedFilesMarkedForCopy: '0',
                formattedFilesMarkedForRemove: '0',
                formattedFilesRemoved: null,
                formattedFilesScanned: '0',
                host: 'nfsServer',
                lastDuration: '20 hours',
                lastMessageTime: '-',
                lastTransferred: 'No previous copy',
                lastTransferredHeaderText: 'Initial Copy | Duration: 20 hours',
                progress: '(94%)',
                showUnknown: false,
                source: {
                    nfs: {
                        'export': '/data1',
                        host: '1.1.1.1'
                    },
                    protocol: 'nfs'
                },
                status: 'RUNNING',
                statusText: 'Sync In Progress',
                sync: {
                    autoSync: 'ON',
                    inDays: 0,
                    inHours: 0,
                    inMinutes: 5
                },
                target: {
                    protocol: 's3',
                    s3: {
                        bucket: 'small'
                    }
                },
                type: 'Initial Copy',
                failureMessage: undefined,
                tags: undefined
            },
            lastCopyMessage: {},
            lastRequests: [],
            lastScanMessage: {},
            lastSyncMessage: undefined
        });

        test("should update inProgress", () => {
            const state = syncReducer(withAuth(), {
                type: actions.GET_RELATIONSHIPS,
                status: actions.STATUS_PROGRESS,
                id: 1
            });

            expect(state._relationships).toEqual({
                failed: false,
                error: undefined,
                inProgress: true,
                succeeded: false,
                id: 1,
                firstTime: true,
                data: undefined,
                requestId: undefined
            })
        });

        test("should update failed", () => {
            const state = syncReducer(withAuth(), {
                type: actions.GET_RELATIONSHIPS,
                status: actions.STATUS_ERROR,
                id: 1
            });

            expect(state._relationships).toEqual({
                failed: true,
                inProgress: false,
                succeeded: false,
                data: undefined,
                id: 1,
                error: undefined,
                requestId: undefined
            })
        });

        test("should update success", () => {
            const state = syncReducer(withAuth(), {
                type: actions.GET_RELATIONSHIPS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                payload: [nfsToS3RelationshipResponse]
            });

            expect(state._relationships).toEqual({
                failed: false,
                inProgress: false,
                succeeded: true,
                error: undefined,
                id: 1,
                data: [nfsToS3RelationshipResult]
            })
        });
    });

    describe("Start wizard", () => {
        test("should set the _wizard state to the payload", () => {
            const state = callSyncReducer(withAuth(), {
                type: actions.START_WIZARD,
                payload: {name: "s3-nfs", stepNamesByWizardType, stepsMapping: stepNameToPathName}
            }, {params: {wizardType: "s3-nfs"}});

            expect(state._wizard).toEqual({
                name: "s3-nfs",
                stepNames: ["Data Broker", "S3 Bucket", "NFS Server", "Directories", "Review"],
                stepNamesByWizardType: {
                    "cifs-s3": ["CIFS Server", "Data Broker", "Shares", "S3 Bucket", "Review"],
                    "cifs-sg": ["CIFS Server", "Data Broker", "Shares", "SG Server", "SG Bucket", "Review"],
                    "nfs-s3": ["NFS Server", "Data Broker", "Directories", "S3 Bucket", "Relationship Tags", "Review"],
                    "nfs-sg": ["NFS Server", "Data Broker", "Directories", "SG Server", "SG Bucket", "Review"],
                    "s3-cifs": ["Data Broker", "S3 Bucket", "CIFS Server", "Shares", "Review"],
                    "s3-nfs": ["Data Broker", "S3 Bucket", "NFS Server", "Directories", "Review"]
                },
                stepsMapping: {
                    "CIFS Server": "cifs",
                    "Data Broker": "agents",
                    "Directories": "exports",
                    "NFS Server": "nfs",
                    "Relationship Tags": "tags",
                    "Review": "review",
                    "S3 Bucket": "buckets",
                    "SG Bucket": "sgbuckets",
                    "SG Server": "sg",
                    "Shares": "shares"
                },
                sync: {},
                wizardType: "s3-nfs",
                sourceType: "storage",
                targetType: "fileSystem"
            })
        })
    });

    describe("Started wizard step", () => {
        test("should add the step name to the steps portion of wizard", () => {
            const wizardStartedState = callSyncReducer(withAuth(), {
                type: actions.START_WIZARD,
                payload: {name: "s3-nfs", stepNamesByWizardType, stepsMapping: stepNameToPathName}
            }, {params: {wizardType: "s3-nfs"}});
            const state = callSyncReducer(wizardStartedState, {
                type: actions.STARTED_WIZARD_STEP,
                payload: "Data Broker"
            });

            expect(state._wizard).toEqual({
                isSourceStep: false,
                urlParams: {},
                name: "s3-nfs",
                stepNamesByWizardType: {
                    "cifs-s3": ["CIFS Server", "Data Broker", "Shares", "S3 Bucket", "Review"],
                    "cifs-sg": ["CIFS Server", "Data Broker", "Shares", "SG Server", "SG Bucket", "Review"],
                    "nfs-s3": ["NFS Server", "Data Broker", "Directories", "S3 Bucket", "Relationship Tags", "Review"],
                    "nfs-sg": ["NFS Server", "Data Broker", "Directories", "SG Server", "SG Bucket", "Review"],
                    "s3-cifs": ["Data Broker", "S3 Bucket", "CIFS Server", "Shares", "Review"],
                    "s3-nfs": ["Data Broker", "S3 Bucket", "NFS Server", "Directories", "Review"]
                },
                stepsMapping: {
                    "CIFS Server": "cifs",
                    "Data Broker": "agents",
                    "Directories": "exports",
                    "NFS Server": "nfs",
                    "Relationship Tags": "tags",
                    "Review": "review",
                    "S3 Bucket": "buckets",
                    "SG Bucket": "sgbuckets",
                    "SG Server": "sg",
                    "Shares": "shares"
                },
                sync: {},
                wizardType: "s3-nfs",
                sourceType: "storage",
                targetType: "fileSystem",
                stepNames: ["Data Broker", "S3 Bucket", "NFS Server", "Directories", "Review"],
                currentStep: "Data Broker",
                steps: {
                    "Data Broker": {started: true}
                }
            })
        });
    });

    describe("clear wizard params", () => {
        const urlParams = {dataBrokerId: "5a0199eb2f5d3f423c0b0520",
            exportPath:"---252Fdata1---252Ffolder1",
            nfsServer:"1.2.2.2",
            rootExport:"---252Fdata1",
            wizardType:"nfs-s3"};

        const buildStateWithWizardParams = () => withAuth()
            .setIn(["_wizard","urlParams"],urlParams)
            .setIn(["_routing", "location","pathname"],
        "/sync/nfs-s3/nfs/1---252E1---252E2---252E1/agents/5a0199eb2f5d3f423c0b0520/exports/---252Fdata1---252Ffolder1/---252Fdata1/30/buckets/large/tags");

        const state1 = syncReducer(buildStateWithWizardParams(), {
            type: actions.CLEAR_URL_PARAMS,
            payload: {changedParam: consts.SELECTED_PARAMS_NFS_SERVER, newParamVal:"1.3.2.2", objKey: ["_routing", "decodedParams"], without: ["exportPath", "rootExport"]}
        });

        test("should clear wizard and routing params", () => {
            expect(state1._wizard).toEqual({"urlParams": undefined});
            expect(state1._routing.decodedParams).toBeUndefined()
        });
        const state2 = syncReducer(buildStateWithWizardParams(), {
            type: actions.CLEAR_URL_PARAMS,
            payload: {changedParam: consts.SELECTED_PARAMS_NFS_SERVER, newParamVal:"1.3.2.2", objKey: ["_wizard"], without: ["selectedExport"]}
        });

        test("should clear only wizard and not routing params", () => {

            expect(state2._wizard).toEqual({
                "selectedExport": undefined,
                "urlParams": undefined
            });
            expect(state2._routing.decodedParams).toEqual(undefined)
        });

        const state3 = syncReducer(buildStateWithWizardParams(), {
            type: actions.CLEAR_URL_PARAMS,
            payload: {changedParam: consts.SELECTED_PARAMS_NFS_SERVER, newParamVal:"1.2.2.2", params: consts.EXPORTS_CLEAR_PATHS}
        });

        test("should not clear params, nfs is the same", () => {

            expect(state3._wizard).toEqual({
                "selectedExport": undefined,
                "urlParams": urlParams
            });
            expect(state3._routing.decodedParams).toEqual(undefined)
        });
});

    describe("Exited wizard step", () => {
        test("should remove the step name from the wizard steps", () => {
            const wizardStartedState = callSyncReducer(withAuth(), {
                type: actions.START_WIZARD,
                payload: {name: "s3-nfs", stepNamesByWizardType, stepsMapping: stepNameToPathName}
            }, {params: {wizardType: "s3-nfs"}});
            const wizardStep1StartedState = callSyncReducer(wizardStartedState, {
                type: actions.STARTED_WIZARD_STEP,
                payload: "Data Broker"
            });
            const wizardStep2StartedState = callSyncReducer(wizardStep1StartedState, {
                type: actions.STARTED_WIZARD_STEP,
                payload: "S3 Bucket"
            });
            const state = callSyncReducer(wizardStep2StartedState, {
                type: actions.EXITED_WIZARD_STEP,
                payload: "S3 Bucket"
            });

            expect(state._wizard).toEqual({
                isSourceStep: false,
                urlParams: {},
                name: "s3-nfs",
                stepNamesByWizardType: {
                    "cifs-s3": ["CIFS Server", "Data Broker", "Shares", "S3 Bucket", "Review"],
                    "cifs-sg": ["CIFS Server", "Data Broker", "Shares", "SG Server", "SG Bucket", "Review"],
                    "nfs-s3": ["NFS Server", "Data Broker", "Directories", "S3 Bucket", "Relationship Tags", "Review"],
                    "nfs-sg": ["NFS Server", "Data Broker", "Directories", "SG Server", "SG Bucket", "Review"],
                    "s3-cifs": ["Data Broker", "S3 Bucket", "CIFS Server", "Shares", "Review"],
                    "s3-nfs": ["Data Broker", "S3 Bucket", "NFS Server", "Directories", "Review"]
                },
                stepsMapping: {
                    "CIFS Server": "cifs",
                    "Data Broker": "agents",
                    "Directories": "exports",
                    "NFS Server": "nfs",
                    "Relationship Tags": "tags",
                    "Review": "review",
                    "S3 Bucket": "buckets",
                    "SG Bucket": "sgbuckets",
                    "SG Server": "sg",
                    "Shares": "shares"
                },
                sync: {},
                wizardType: "s3-nfs",
                sourceType: "storage",
                targetType: "fileSystem",
                currentStep: "Data Broker",
                stepNames: ["Data Broker", "S3 Bucket", "NFS Server", "Directories", "Review"],
                steps: {
                    "Data Broker": {
                        started: true
                    }
                }
            })
        })
    });

    describe("Update Exports", () => {

        const host = '1.1.1.1';
        const dataBrokerId = 1;
        const extra = {host, dataBrokerId};
        const path = 'data1';

        const buildStateWithoutExportsFolders = () => withAuth()
            .set("_showExports", {
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {}
                    },
                    id: 1
                }
            });

        const buildStateWithExportsFolders = () => withAuth()
            .set("_showExports", {
                "1---252E1---252E1---252E1": {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {
                            data: [
                                {
                                    actualPath: '/' + path,
                                    host,
                                    path
                                }
                            ]
                        }
                    },
                    data: [
                        {
                            actualPath: '/' + path,
                            host,
                            path
                        }
                    ],
                    id: 1
                }
            });

        test("should update retrievedAll and failedRetrieve upon failure in show-exports", () => {

            const state = syncReducer(buildStateWithoutExportsFolders(), {
                type: actions.UPDATE_EXPORTS, payload: {
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "FAILED",
                        host
                    }
                },
                status: actions.STATUS_ERROR, id: 1, extra
            });


            expect(state._showExports).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        failedRetrieve: true,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            "-": {
                                retrievedAll: true
                            }
                        },
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });

        test("should update exports upon status running in show-exports and the state doesnt have any exports", () => {

            const state = syncReducer(buildStateWithoutExportsFolders(), {
                type: actions.UPDATE_EXPORTS, payload: {
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host,
                        paths: [path]
                    }
                },
                status: actions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showExports).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        retrievedAll: true,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path
                                    }
                                ],
                                retrievedAll: true
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            }
                        ],
                        id: 1
                    }
                }
            );
        });

        test("should update exports upon status running in show-exports and the state has previous exports", () => {

            const state = syncReducer(buildStateWithExportsFolders(), {
                type: actions.UPDATE_EXPORTS, payload: {
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host,
                        paths: ['data2']
                    }
                },
                status: actions.STATUS_SUCCESS, id: 1, extra
            });

            expect(state._showExports).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        retrievedAll: true,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path
                                    },
                                    {
                                        actualPath: "/data2",
                                        host,
                                        path: "data2"
                                    }
                                ],
                                retrievedAll: true,
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            },
                            {
                                actualPath: "/data2",
                                host,
                                path: "data2"
                            }
                        ],
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll and export upon status DONE in show-exports and the state doesnt have any exports", () => {

            const state = syncReducer(buildStateWithoutExportsFolders(), {
                type: actions.UPDATE_EXPORTS, payload: {
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host,
                        paths: []
                    }
                },
                status: actions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showExports).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        data: [],
                        folders: {
                            "-": {
                                data: [],
                                retrievedAll: true
                            }
                        },
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll and export upon status DONE in show-exports and the state has previous exports", () => {

            const state = syncReducer(buildStateWithExportsFolders(), {
                type: actions.UPDATE_EXPORTS, payload: {
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host,
                        paths: []
                    }
                },
                status: actions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showExports).toEqual({
                    "1---252E1---252E1---252E1": {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path
                                    }
                                ],
                                retrievedAll: true
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            }
                        ],
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });

        test("should return state with unhandledMessages - _showExports is undefined", () => {
            const action = {
                type: actions.UPDATE_EXPORTS, payload: {
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host: '2.2.2.2'
                    }
                },
                status: actions.STATUS_SUCCESS, id: 1, extra
            };

            const state = syncReducer(withAuth(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(withAuth().set("unhandledMessages", [action]));
        });

        test("should return state with unhandledMessages - host is not like _showExports.host", () => {

            const action = {
                type: actions.UPDATE_EXPORTS, payload: {
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host: '2.2.2.2'
                    }, status: actions.STATUS_SUCCESS, id: 1, extra
                }
            };

            const state = syncReducer(buildStateWithoutExportsFolders(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(buildStateWithoutExportsFolders().set("unhandledMessages", [action]));

        });

    });


    describe("Update SG Buckets", () => {
        const host = '1.1.1.1';
        const port = 8080;
        const accessKey = 'a';
        const secretKey = 'b';

        const stateWithShowSGBucketsField = () => withAuth().set("_showSGBuckets", { '1.1.1.1' : {
            failed: false,
            inProgress: false,
            error: undefined,
            requestId: undefined,
            succeeded: true,
            id: 1,
            folders: {
                "-": {}
            },
            host,
            port,
            accessKey,
            secretKey

        }});

        test("should return state with unhandledMessages - _showSGBuckets is undefined", () => {

            const action = {
                type: actions.UPDATE_SG_BUCKETS, payload: { data: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS]: {
                        buckets: [{name: 'large'},
                            {name: 'small'}],
                        host,
                        port,
                        status: 'DONE'
                    }
                }}, status: actions.STATUS_SUCCESS, id: 1
            };

            const state = syncReducer(withAuth(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(withAuth().set("unhandledMessages", [action]));
        });

        test("should return retrievedAll true and not update the list - action status is failed", () => {

            const state = syncReducer(stateWithShowSGBucketsField(), {
                type: actions.UPDATE_SG_BUCKETS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS]: {
                        status: 'FAILED',
                        host: '1.1.1.1'
                    }, status: actions.STATUS_ERROR, id: 1
                }
            });
            expect(state._showSGBuckets).toEqual({ '1.1.1.1': {

                failed: false,
                inProgress: false,
                error: undefined,
                requestId: undefined,
                succeeded: true,
                id: 1,
                folders: {
                    "-": {retrievedAll: true}
                },
                retrievedAll: true,
                failedRetrieve: true,
                host,
                port,
                accessKey,
                secretKey
            }});
        });

        test("should return retrievedAll true and check list was updated - action status is DONE", () => {

            const state = syncReducer(stateWithShowSGBucketsField(), {
                type: actions.UPDATE_SG_BUCKETS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS]: {
                        buckets: [{name: 'large'},
                            {name: 'small'}],
                        host,
                        port,
                        status: 'DONE'
                    }
                }
                , status: actions.STATUS_SUCCESS, id: 1
            });
            expect(state._showSGBuckets).toEqual({ '1.1.1.1' : {

                failed: false,
                inProgress: false,
                error: undefined,
                requestId: undefined,
                succeeded: true,
                id: 1,
                data: [{actualPath: 'large', path: 'large', name: 'large'},
                    {actualPath: 'small', path: 'small', name: 'small'}],
                folders: {
                    "-": {
                        data: [{actualPath: 'large', path: 'large', name: 'large'},
                            {actualPath: 'small', path: 'small', name: 'small'}],
                        retrievedAll: true
                    }
                },
                retrievedAll: true,
                failedRetrieve: false,
                host,
                port,
                accessKey,
                secretKey
            }});
        });

        test("should return retrievedAll true and check list was updated - no buckets", () => {

            const state = syncReducer(stateWithShowSGBucketsField(), {
                type: actions.UPDATE_SG_BUCKETS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS]: {
                        buckets: [], host, port, status: 'DONE'
                    }, status: actions.STATUS_SUCCESS, id: 1
                }
            });
            expect(state._showSGBuckets).toEqual({ '1.1.1.1': {
                    failed: false,
                    inProgress: false,
                    error: undefined,
                    requestId: undefined,
                    succeeded: true,
                    id: 1,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    failedRetrieve: false,
                    host,
                    port,
                    accessKey,
                    secretKey
                }}
                );
        });
    });

    describe("Update Buckets", () => {
        const dataBrokerId = "1";
        const stateWithShowBucketsField = () => withAuth().set("_showBuckets", {
            "1" : {
            failed: false,
            inProgress: false,
            error: undefined,
            requestId: undefined,
            succeeded: true,
            id: 1,
            folders: {
                "-": {}
            }
        }
        } );

        test("should return state with unhandledMessages - _showBuckets is undefined", () => {

            const action = {
                type: actions.UPDATE_BUCKETS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_BUCKETS]: [{
                        name: 'large',
                        region: {displayName: 'EU (Frankfurt)', name: 'eu-central-1'},
                        tags: []
                    },
                        {name: 'small', region: {displayName: 'US West (Oregon)', name: 'us-west-2'}, tags: []}]
                }, status: actions.STATUS_SUCCESS, id: 1
            };

            const state = syncReducer(withAuth(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(withAuth().set("unhandledMessages", [action]));
        });

        test("should return retrievedAll true and not update the list - action status is failed", () => {

            const state = syncReducer(stateWithShowBucketsField(), {
                type: actions.UPDATE_BUCKETS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_BUCKETS]: {
                        status: 'FAILED'
                    }
                }, status: actions.STATUS_ERROR, id: 1
            });
            expect(state._showBuckets).toEqual({
                "1" : {
                    failed: false,
                    inProgress: false,
                    error: undefined,
                    requestId: undefined,
                    succeeded: true,
                    id: 1,
                    folders: {
                        "-": {retrievedAll: true}
                    },
                    retrievedAll: true,
                    failedRetrieve: true
                }
                }
                );
        });

        test("should return retrievedAll true and check list was updated - action status is DONE", () => {

            const state = syncReducer(stateWithShowBucketsField(), {
                type: actions.UPDATE_BUCKETS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_BUCKETS]: [{
                        name: 'large',
                        region: {displayName: 'EU (Frankfurt)', name: 'eu-central-1'},
                        tags: []
                    },
                        {name: 'small', region: {displayName: 'US West (Oregon)', name: 'us-west-2'}, tags: []}]
                }, status: actions.STATUS_SUCCESS, id: 1
            });
            expect(state._showBuckets).toEqual({
                "1" : {
                    failed: false,
                    inProgress: false,
                    error: undefined,
                    requestId: undefined,
                    succeeded: true,
                    id: 1,
                    data: [{
                        actualPath: 'large',
                        path: 'large',
                        name: 'large',
                        region: {displayName: 'EU (Frankfurt)', name: 'eu-central-1'},
                        tags: []
                    },
                        {
                            actualPath: 'small',
                            path: 'small',
                            name: 'small',
                            region: {displayName: 'US West (Oregon)', name: 'us-west-2'},
                            tags: []
                        }],
                    folders: {
                        "-": {
                            data: [{
                                actualPath: 'large',
                                path: 'large',
                                name: 'large',
                                region: {displayName: 'EU (Frankfurt)', name: 'eu-central-1'},
                                tags: []
                            },
                                {
                                    actualPath: 'small',
                                    path: 'small',
                                    name: 'small',
                                    region: {displayName: 'US West (Oregon)', name: 'us-west-2'},
                                    tags: []
                                }],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    failedRetrieve: false

                }
                }
                );
        });

        test("should return retrievedAll true and check list was updated - no buckets", () => {

            const state = syncReducer(stateWithShowBucketsField(), {
                type: actions.UPDATE_BUCKETS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_BUCKETS]: []
                }, status: actions.STATUS_SUCCESS, id: 1
            });
            expect(state._showBuckets).toEqual({
                "1" : {
                    failed: false,
                    inProgress: false,
                    error: undefined,
                    requestId: undefined,
                    succeeded: true,
                    id: 1,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    failedRetrieve: false
                }
                }
                );
        });
    });

    describe("Update Buckets Folders", () => {
        const dataBrokerId = 1;
        const bucket = 'thisBucket';
        const path = 'medium2';
        const stateWithShowBucketsFieldWithBuckets = () => withAuth().setIn(["_showBuckets", "1"], {
            error: undefined,
            failed: false,
            inProgress: false,
            succeeded: true,
            data: [{
                actualPath: bucket,
                name: bucket,
                path: bucket,
                region: {
                    displayName: 'US WEST (Oregon)',
                    name: 'us-west-2'
                },
                tags: []
            }],
            folders: {
                '-': {
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket,
                        region: {
                            displayName: 'US WEST (Oregon)',
                            name: 'us-west-2'
                        },
                        tags: []
                    }],
                    retrievedAll: true
                },
                "thisBucket": {
                    data: [],
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    requestId: undefined,
                    id: 1
                }
            },
            retrievedAll: true,
            id: 1
        });


        const stateWithBucketsAndExtraListFolders = () => withAuth().setIn(["_showBuckets", "1"], {
            error: undefined,
            failed: false,
            inProgress: false,
            succeeded: true,
            data: [{
                actualPath: bucket,
                name: bucket,
                path: bucket,
                region: {
                    displayName: 'US WEST (Oregon)',
                    name: 'us-west-2'
                },
                tags: []
            }],
            folders: {
                '-': {
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket,
                        region: {
                            displayName: 'US WEST (Oregon)',
                            name: 'us-west-2'
                        },
                        tags: []
                    }],
                    retrievedAll: true
                },
                "thisBucket": {
                    data: [{actualPath: `${bucket}/${path}`, bucket, name: path, path}],
                    failed: false,
                    inProgress: false,
                    retrievedAll: true,
                    succeeded: true,
                    requestId: undefined,
                    id: 1
                },
                "thisBucket/medium2": {
                    data: [],
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    requestId: undefined,
                    id: 1
                }
            },
            retrievedAll: true,
            id: 1
        });

        test("should return state with unhandledMessages - _showBuckets is undefined", () => {
            const data = {
                path: bucket,
                status: 'DONE',
                bucket
            };
            const action = {
                type: actions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            };

            const state = syncReducer(withAuth(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(withAuth().set("unhandledMessages", [action]));
        });

        test("should return undefined and the same state that was sent- state._showBuckets.data is not exist", () => {
            const show =
                {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                };

            const data = {
                path: bucket,
                status: 'DONE',
                bucket
            };
            const action = {
                type: actions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            };

            const state = syncReducer(withAuth().setIn(["_showBuckets", "1"], show),action );

            expect(state._showBuckets["1"]).toEqual(show);
        });

        test("Should return the same state that was sent - state._showBuckets.data = []", () => {
            const prevState = withAuth()
                .setIn(["_showBuckets","1"], {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                });

            const data = {
                path: bucket,
                status: 'DONE',
                bucket
            };
            const action = {
                type: actions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            };
            const currState = syncReducer(prevState, action);
            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent - bucket name does not exist in data", () => {

            const prevState = withAuth()
                .setIn(["_showBuckets","1"], {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                });

            const data = {
                path: 'bucket1',
                status: 'DONE',
                bucket: 'bucket1'
            };

            const state = syncReducer(prevState, {
                type: actions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            });
            expect(state._showBuckets).toEqual(prevState._showBuckets);
        });

        test("Should return the same state that was sent - path does not exist in folders", () => {

            const prevState = withAuth()
                .setIn(["_showBuckets", "1" ], {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                });
            const data = {
                path: `${bucket}/bucket11`,
                status: 'DONE',
                bucket
            };

            const state = syncReducer(prevState, {
                type: actions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            });
            expect(state._showBuckets).toEqual(prevState._showBuckets);

        });


        test("should return retrievedAll true and check list was updated - action status is DONE", () => {
            const data = {
                status: 'DONE',
                list: ['medium2'],
                bucket
            };

            const state = syncReducer(stateWithShowBucketsFieldWithBuckets(), {
                type: actions.UPDATE_BUCKETS_FOLDERS,
                payload: {
                    dataBrokerId,
                    status: undefined,
                    id: 1,
                    data,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data
                }
            });
            expect(state._showBuckets["1"]).toEqual({

                failed: false,
                inProgress: false,
                error: undefined,
                succeeded: true,
                id: 1,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket,
                    region: {
                        displayName: 'US WEST (Oregon)',
                        name: 'us-west-2'
                    },
                    tags: []
                }],
                folders: {
                    "-": {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket,
                            region: {
                                displayName: 'US WEST (Oregon)',
                                name: 'us-west-2'
                            },
                            tags: []
                        }],
                        retrievedAll: true
                    },
                    'thisBucket': {
                        data: [{actualPath: 'thisBucket/medium2', bucket, name: 'medium2', path: 'medium2'}],
                        failed: false,
                        inProgress: false,
                        id: 1,
                        retrievedAll: true,
                        requestId: undefined,
                        succeeded: true
                    }
                },
                retrievedAll: true
            });
        });

        test("should return retrievedAll true and check list was updated - action status is DONE - deep dive", () => {
            const data = {
                prefix: '/medium2',
                status: 'DONE',
                list: ['medium3'],
                bucket
            };

            const state = syncReducer(stateWithBucketsAndExtraListFolders(), {
                    type: actions.UPDATE_BUCKETS_FOLDERS,
                    payload: {
                        dataBrokerId,
                        status: undefined,
                        id: 1,
                        data,
                        [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data
                    }
                }
            );
            expect(state._showBuckets["1"]).toEqual({
                error: undefined,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket,
                    region: {
                        displayName: 'US WEST (Oregon)',
                        name: 'us-west-2'
                    },
                    tags: []
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket,
                            region: {
                                displayName: 'US WEST (Oregon)',
                                name: 'us-west-2'
                            },
                            tags: []
                        }],
                        retrievedAll: true
                    },
                    "thisBucket": {
                        data: [{actualPath: `${bucket}/${path}`, bucket, name: path, path}],
                        failed: false,
                        retrievedAll: true,
                        inProgress: false,
                        succeeded: true,
                        requestId: undefined,
                        id: 1
                    },
                    "thisBucket/medium2": {
                        data: [
                            {
                                actualPath: "thisBucket/medium2/medium3",
                                bucket,
                                name: "medium3",
                                path: "medium3"
                            }
                        ],
                        failed: false,
                        retrievedAll: true,
                        inProgress: false,
                        succeeded: true,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                id: 1
            });
        });


        test("should return retrievedAll true and check list was updated - action status is FAILED", () => {
            const data = {
                path: bucket,
                status: 'FAILED',
                bucket,
                list: ['medium2']
            };

            const state = syncReducer(stateWithShowBucketsFieldWithBuckets(), {
                type: actions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            });
            expect(state._showBuckets["1"]).toEqual({

                failed: false,
                inProgress: false,
                error: undefined,
                succeeded: true,
                id: 1,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket,
                    region: {
                        displayName: 'US WEST (Oregon)',
                        name: 'us-west-2'
                    },
                    tags: []
                }],
                folders: {
                    "-": {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket,
                            region: {
                                displayName: 'US WEST (Oregon)',
                                name: 'us-west-2'
                            },
                            tags: []
                        }],
                        retrievedAll: true
                    },
                    'thisBucket': {
                        data: [],
                        failed: false,
                        inProgress: false,
                        id: 1,
                        retrievedAll: true,
                        requestId: undefined,
                        succeeded: true
                    }
                },
                retrievedAll: true
            });
        });
    });

    describe("Update SG Buckets Folders", () => {
        const dataBrokerId = 1;
        const bucket = 'thisBucket';
        const port = 8080;
        const accessKey = 'a';
        const secretKey = 'a';
        const host = '1.1.1.1';
        const path = 'medium2';

        const stateWithShowSGBucketsFieldWithBuckets = () => withAuth().set("_showSGBuckets", { '1.1.1.1' : {
            error: undefined,
            failed: false,
            inProgress: false,
            port,
            accessKey,
            secretKey,
            succeeded: true,
            data: [{
                actualPath: bucket,
                name: bucket,
                path: bucket
            }],
            folders: {
                '-': {
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket
                    }],
                    retrievedAll: true
                },
                "thisBucket": {
                    data: [],
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    requestId: undefined,
                    id: 1
                }
            },
            retrievedAll: true,
            id: 1
        }});


        const stateWithSGBucketsAndExtraListFolders = () => withAuth().set("_showSGBuckets", { '1.1.1.1': {
                error: undefined,
                failed: false,
                inProgress: false,
                port,
                accessKey,
                secretKey,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    "thisBucket": {
                        data: [{actualPath: 'thisBucket/medium2', bucket, name: 'medium2', path: 'medium2'}],
                        failed: false,
                        inProgress: false,
                        id: 1,
                        retrievedAll: true,
                        requestId: undefined,
                        succeeded: true
                    },
                    "thisBucket/medium2": {
                        data: [],
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                id: 1
            }});

        test("should return state with unhandledMessages - _showSGBuckets is undefined", () => {
            const data = {
                path: bucket,
                status: 'DONE',
                bucket,
                host
            };
            const action = {
                type: actions.UPDATE_SG_BUCKETS_FOLDERS,
                payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: data,
                    status: undefined,
                    id: 1,
                    data
                }
            };

            const state = syncReducer(withAuth(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(withAuth().set("unhandledMessages", [action]));
        });

        test("should return undefined and the same state that was sent- state._showSGBuckets.data is not exist", () => {
            const show =
                { '1.1.1.1': {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                }};

            const state = syncReducer(withAuth().set("_showSGBuckets", show), {
                type: actions.UPDATE_SG_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: {
                        path: bucket,
                        status: 'DONE',
                        bucket,
                    }, status: undefined, id: 1,
                    data: {
                        path: bucket,
                        status: 'DONE',
                        bucket,
                        host
                    }
                }
            });

            expect(state._showSGBuckets).toEqual(show);
            expect(state).toEqual(withAuth().set("_showSGBuckets", show));
        });

        test("Should return the same state that was sent - state._showSGBuckets.data = []", () => {
            const prevState = withAuth()
                .set("_showSGBuckets", { '1.1.1.1': {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                }});
            const currState = syncReducer(prevState, {
                type: actions.UPDATE_SG_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: {
                        path: bucket,
                        status: 'DONE',
                        bucket,
                    }, status: undefined, id: 1
                    ,
                    data: {
                        path: bucket,
                        status: 'DONE',
                        bucket,
                        host
                    }
                }
            });


            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent - bucket name does not exist in data", () => {

            const prevState = withAuth()
                .set("_showSGBuckets", { '1.1.1.1': {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                }});

            const state = syncReducer(prevState, {
                type: actions.UPDATE_SG_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: {
                        path: 'bucket1',
                        status: 'DONE',
                        bucket: 'bucket1',
                    }, status: undefined, id: 1
                    ,
                    data: {
                        path: 'bucket1',
                        status: 'DONE',
                        bucket: 'bucket1',
                        host
                    }
                }
            });
            expect(state._showSGBuckets).toEqual(prevState._showSGBuckets);

        });

        test("Should return the same state that was sent - path does not exist in folders", () => {

            const prevState = withAuth()
                .set("_showSGBuckets", { '1.1.1.1': {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                }});

            const state = syncReducer(prevState, {
                type: actions.UPDATE_SG_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: {
                        path: `${bucket}/bucket11`,
                        status: 'DONE',
                        bucket,
                    }, status: undefined, id: 1
                    ,
                    data: {
                        path: `${bucket}/bucket11`,
                        status: 'DONE',
                        bucket,
                        host
                    }
                }
            });
            expect(state._showSGBuckets).toEqual(prevState._showSGBuckets);

        });


        test("should return retrievedAll true and check list was updated - action status is DONE", () => {
            const data = {
                status: 'DONE',
                bucket,
                list: ['medium2'],
                host,
                port
            };
            const action = {
                type: actions.UPDATE_SG_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: data, status: undefined, id: 1,
                    data
                }
            };

            const state = syncReducer(stateWithShowSGBucketsFieldWithBuckets(), action);
            expect(state._showSGBuckets).toEqual({ '1.1.1.1': {
                accessKey,
                port,
                secretKey,
                failed: false,
                inProgress: false,
                error: undefined,
                succeeded: true,
                id: 1,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket

                }],
                folders: {
                    "-": {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket

                        }],
                        retrievedAll: true
                    },
                    'thisBucket': {
                        data: [{actualPath: 'thisBucket/medium2', bucket, name: 'medium2', path: 'medium2'}],
                        failed: false,
                        inProgress: false,
                        id: 1,
                        retrievedAll: true,
                        requestId: undefined,
                        succeeded: true
                    }
                },
                retrievedAll: true
            }});
        });


        test("should return retrievedAll true and check list was updated - action status is FAILED", () => {

            const data = {
                status: 'FAILED',
                bucket,
                list: ['medium2'],
                host,
                port
            };
            const action = {
                type: actions.UPDATE_SG_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: data, status: undefined,
                    id: 1,
                    data
                }
            };

            const state = syncReducer(stateWithShowSGBucketsFieldWithBuckets(), action);
            expect(state._showSGBuckets).toEqual({ '1.1.1.1': {
                accessKey,
                port,
                secretKey,
                failed: false,
                inProgress: false,
                error: undefined,
                succeeded: true,
                id: 1,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket

                }],
                folders: {
                    "-": {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket

                        }],
                        retrievedAll: true
                    },
                    'thisBucket': {
                        data: [],
                        failed: false,
                        inProgress: false,
                        id: 1,
                        retrievedAll: true,
                        requestId: undefined,
                        succeeded: true
                    }
                },
                retrievedAll: true
            }});
        });

        test("should return retrievedAll true and check list was updated - action status is DONE -  deep dive", () => {
            const data = {
                status: 'DONE',
                list: ['medium3'],
                prefix: `/${path}`,
                bucket,
                host,
                port
            };

            const state = syncReducer(stateWithSGBucketsAndExtraListFolders(), {
                type: actions.UPDATE_SG_BUCKETS_FOLDERS,
                payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: data,
                    status: undefined,
                    id: 1,
                    data
                }
            });
            expect(state._showSGBuckets).toEqual({ '1.1.1.1': {
                accessKey,
                secretKey,
                port,
                error: undefined,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    "thisBucket": {
                        data: [{actualPath: `${bucket}/${path}`, bucket, name: path, path}],
                        failed: false,
                        retrievedAll: true,
                        inProgress: false,
                        succeeded: true,
                        requestId: undefined,
                        id: 1
                    },
                    "thisBucket/medium2": {
                        data: [
                            {
                                actualPath: "thisBucket/medium2/medium3",
                                bucket,
                                name: "medium3",
                                path: "medium3"
                            }
                        ],
                        failed: false,
                        retrievedAll: true,
                        inProgress: false,
                        succeeded: true,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                id: 1
            }});
        });

    });


    describe("Add BYOL License", () => {

        test("should add a license to license array", () => {
            const state = syncReducer(withAuth(), {
                type: actions.ADD_BYOL_LICENSE,
                status: actions.STATUS_SUCCESS,
                id: 1,
                payload: {
                    expirationDate: 0,
                    reservedRelationships: 20,
                    serialNumber: "991"
                }
            });

            expect(state._licenses.data.licenses).toEqual([
                    {
                        expirationDate: 0,
                        reservedRelationships: 20,
                        serialNumber: "991"
                    }
                ]
            );
        });

    });

    describe("Get License Settings", () => {

        const initialState = withAuth().set("auth", {succeeded: true})
            .set("_licenses", {succeeded: true, data: {licenses: []}});

        const onTrialState = initialState.setIn(["auth", "data", "daysLeftForTrial"], 7);
        const trialEndedState = initialState.setIn(["auth", "data", "daysLeftForTrial"], 0);

        const onTrialNotSubscribedState = onTrialState.setIn(["auth", "data", "subscriptionStatus"], 'unsubscribed-init');
        const trialEndedNotSubscribedState = trialEndedState.setIn(["auth", "data", "subscriptionStatus"], 'unsubscribed-init');
        const initialStateSubscribed = onTrialState.setIn(["auth", "data", "subscriptionStatus"], consts.SAAS_REGISTRATION_STATUS_SUBSCRIBE_SUCCESS);

        const reduce = (state, payload) => {
            return syncReducer(state, {
                type: actions.GET_LICENSE_SETTINGS,
                status: actions.STATUS_SUCCESS,
                id: 1,
                payload
            });
        };

        test("should construct the available license for Free Trial", () => {
            const payload = {
                freeTrial: true,
                existingRelationships: 2
            };
            const licenseType = consts.LICENSE_TYPE_FREE_TRIAL;

            const state = reduce(onTrialNotSubscribedState, payload);

            // user doesn't need to pay - still in free trial
            expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {licenseType, needToPay: false}));
        });

        describe("should construct the available license for PayGo", () => {
            const payload = {
                freeTrial: false,
                existingRelationships: 3,
                reservedRelationships: 0,
                totalReservedRelationships: 0,
                chargedRelationships: 3
            };
            const licenseType = consts.LICENSE_TYPE_PAYGO;

            test("when not subscribed and on trial", () => {
                const state = reduce(onTrialNotSubscribedState, payload);

                // user needs to pay - the marketplace charged relationships are not payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {licenseType, needToPay: true}));

                // user should see trial page and not be blocked
                expect(state._ui.showTrialPage).toEqual(true);
                expect(state._ui.blocked).toEqual(false);
            });

            test("when not subscribed and not on trial", () => {
                const state = reduce(trialEndedNotSubscribedState, payload);

                // user needs to pay - the marketplace charged relationships are not payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {licenseType, needToPay: true}));

                // user should not see trial page and be blocked
                expect(state._ui.showTrialPage).toEqual(false);
                expect(state._ui.blocked).toEqual(true);
            });

            test("when subscribed", () => {
                const state = reduce(initialStateSubscribed, payload);

                // user doesn't need to pay - the marketplace charged relationships are payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {
                    licenseType,
                    needToPay: false
                }));

                // user should not see trial page and not be blocked
                expect(state._ui.showTrialPage).toEqual(false);
                expect(state._ui.blocked).toEqual(false);
            });

        });

        describe("should construct the available license for BYOL", () => {
            const payload = {
                freeTrial: false,
                existingRelationships: 19,
                reservedRelationships: 19,
                totalReservedRelationships: 20,
                chargedRelationships: 0
            };
            const licenseType = consts.LICENSE_TYPE_BYOL;

            const addLicenseToState = (state) => state.setIn(["_licenses", "data", "licenses"], ['one license']);

            test("when not subscribed and on trial", () => {
                const stateWithLicense = addLicenseToState(onTrialNotSubscribedState);
                const nextState = reduce(stateWithLicense, payload);

                // user doesn't need to pay - all relationships are within the license capacity
                expect(nextState._licenseSettings.data).toEqual(Object.assign({}, payload, {
                    licenseType,
                    needToPay: false
                }));

                // user should not see trial page and not be blocked
                expect(nextState._ui.showTrialPage).toEqual(false);
                expect(nextState._ui.blocked).toEqual(false);
            });

            test("when not subscribed and not on trial", () => {
                const stateWithLicense = addLicenseToState(trialEndedNotSubscribedState);
                const nextState = reduce(stateWithLicense, payload);

                // user doesn't need to pay - all relationships are within the license capacity
                expect(nextState._licenseSettings.data).toEqual(Object.assign({}, payload, {
                    licenseType,
                    needToPay: false
                }));

                // user should not see trial page and not be blocked
                expect(nextState._ui.showTrialPage).toEqual(false);
                expect(nextState._ui.blocked).toEqual(false);
            });

            test("when subscribed", () => {
                const stateWithLicense = addLicenseToState(initialStateSubscribed);
                const nextState = reduce(stateWithLicense, payload);

                // user doesn't need to pay - all relationships are within the license capacity
                expect(nextState._licenseSettings.data).toEqual(Object.assign({}, payload, {
                    licenseType,
                    needToPay: false
                }));

                // user should not see trial page and not be blocked
                expect(nextState._ui.showTrialPage).toEqual(false);
                expect(nextState._ui.blocked).toEqual(false);
            });
        });

        describe("should construct the available license for PayGo with BYOL", () => {
            const payload = {
                freeTrial: false,
                existingRelationships: 24,
                reservedRelationships: 20,
                totalReservedRelationships: 20,
                chargedRelationships: 4
            };
            const licenseType = consts.LICENSE_TYPE_PAYGO_WITH_BYOL;

            test("when not subscribed and on trial", () => {
                const state = reduce(onTrialNotSubscribedState, payload);

                // user needs to pay - there are marketplace charged relationships that are not payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {licenseType, needToPay: true}));

                // user should see trial page and not be blocked
                expect(state._ui.showTrialPage).toEqual(true);
                expect(state._ui.blocked).toEqual(false);
            });

            test("when not subscribed and not on trial", () => {
                const state = reduce(trialEndedNotSubscribedState, payload);

                // user needs to pay - there are marketplace charged relationships that are not payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {licenseType, needToPay: true}));

                // user should not see trial page and not be blocked, just can't create a relationship
                expect(state._ui.showTrialPage).toEqual(false);
                expect(state._ui.blocked).toEqual(false);
                expect(state._ui.shouldBlockCreatingRelationship).toBeDefined(); // not null or undefined
            });

            test("when subscribed", () => {
                const state = reduce(initialStateSubscribed, payload);

                // user doesn't need to pay - the marketplace charged relationships are payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {
                    licenseType,
                    needToPay: false
                }));

                // user should not see trial page and not be blocked
                expect(state._ui.showTrialPage).toEqual(false);
                expect(state._ui.blocked).toEqual(false);
            });
        });

    });
});

describe("handleErrors", () => {
    beforeEach(() => {
        actions.routesContainer.routes = {};
    });
    test("should return the same state if error is on login", function () {
        const state = syncReducer(withAuth(), {
            "type": "LOGIN_LOCAL",
            "payload": {data: {message: "there was an error"}},
            "status": actions.STATUS_ERROR
        });

        expect(state._generalErrors._errorsMap).toEqual({});
        expect(state._generalErrors._errorsPanelOpen).toEqual(false);
    });

    test("should update the error in the errors map and set errorsPanel to open, and clear it when success for the same message arrives", function () {
        const state = syncReducer(withAuth(), {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": {
                "data": {
                    "message": "Error deleting storage system"
                }
            },
            "status": actions.STATUS_ERROR
        });

        expect(state._generalErrors._errorsMap).toEqual(expect.objectContaining({"DELETE_STORAGE_SYSTEM": expect.anything()}));
        expect(state._generalErrors._errorsMap["DELETE_STORAGE_SYSTEM"]).toEqual(expect.objectContaining({
            id: "DELETE_STORAGE_SYSTEM",
            message: "Error deleting storage system"
        }));
        expect(state._generalErrors._errorsPanelOpen).toEqual(true);

        //now clear the error
        const clearedState = syncReducer(state, {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": {},
            "status": actions.STATUS_SUCCESS
        });
        expect(clearedState._generalErrors._errorsMap).toEqual({});
    });

    test("should network errors properly", function () {
        const state = syncReducer(withAuth(), {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": {status: 504},
            "status": actions.STATUS_ERROR
        });

        expect(state._generalErrors._errorsMap).toEqual(expect.objectContaining({"NETWORK": expect.anything()}));
        expect(state._generalErrors._errorsMap["NETWORK"]).toEqual(expect.objectContaining({
            id: "NETWORK",
            message: "Please check your network connectivity and refresh the browser."
        }));
        expect(state._generalErrors._errorsPanelOpen).toEqual(true);

        //now claer the error on any success
        const clearedState = syncReducer(state, {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": {},
            "status": actions.STATUS_SUCCESS
        });
        expect(clearedState._generalErrors._errorsMap).toEqual({});
    });

    test("should handle client errors properly", function () {
        const state = syncReducer(withAuth(), {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": new Error("Not a network error"),
            "status": actions.STATUS_ERROR
        });

        expect(state._generalErrors._errorsMap).toEqual(expect.objectContaining({"DELETE_STORAGE_SYSTEM": expect.anything()}));
        expect(state._generalErrors._errorsMap["DELETE_STORAGE_SYSTEM"]).toEqual(expect.objectContaining({
            id: "DELETE_STORAGE_SYSTEM",
            message: "Client Error: Not a network error"
        }));
        expect(state._generalErrors._errorsPanelOpen).toEqual(true);

        //now claer the error on any success
        const clearedState = syncReducer(state, {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": {},
            "status": actions.STATUS_SUCCESS
        });
        expect(clearedState._generalErrors._errorsMap).toEqual({});
    });

    test("should remove the right error when there are more than 1 and only one of them is success", function () {
        const state = syncReducer(withAuth(), {
            "type": "FIRST_ERROR",
            "payload": {
                "data": {
                    "message": "Error deleting storage system"
                }
            },
            "status": actions.STATUS_ERROR
        });
        expect(state._generalErrors._errorsMap).toEqual(expect.objectContaining({"FIRST_ERROR": expect.anything()}));

        const state1 = syncReducer(state, {
            "type": "ANOTHER_ERROR",
            "payload": {
                "data": {
                    "message": "this is another error"
                }
            },
            "status": actions.STATUS_ERROR
        });
        expect(state1._generalErrors._errorsMap).toEqual(expect.objectContaining({
            "FIRST_ERROR": expect.anything(),
            "ANOTHER_ERROR": expect.anything()
        }));

        //now clear the first
        const clearedState = syncReducer(state1, {
            "type": "FIRST_ERROR",
            "payload": {},
            "status": actions.STATUS_SUCCESS
        });
        expect(clearedState._generalErrors._errorsMap).not.toEqual(expect.objectContaining({"FIRST_ERROR": expect.anything()}));
    });
});


function testUpdateKeyWithBuiltState(stateKey, actionKey, ignoreSuccess, differentDataIn) {
    return () => describe(`Update ${stateKey} with built state`, () => {
        test("should update inProgress", () => {
            const state = syncReducer(withAuth(), {type: actionKey, status: actions.STATUS_PROGRESS, id: 1, extra: {}});

            expect(state[stateKey]).toEqual({
                id: 1,
                failed: false,
                inProgress: true,
                succeeded: false,
                data: undefined,
                error: undefined,
            })
        });

        test("should update failed", () => {
            const state = syncReducer(withAuth(), {
                type: actions[actionKey],
                payload: new Error("error"),
                status: actions.STATUS_ERROR,
                id: 1,
                extra: {}
            });

            expect(state[stateKey]).toEqual({
                id: 1,
                failed: true,
                inProgress: false,
                succeeded: false,
                data: undefined,
                error: new Error("error"),
            })
        });

        if (!ignoreSuccess) {
            test("should update success", () => {
                const state = syncReducer(withAuth(), {
                    type: actions[actionKey],
                    payload: differentDataIn ? differentDataIn : {something: "something"},
                    status: actions.STATUS_SUCCESS,
                    id: 1,
                    extra: {}
                });

                // console.log(state[stateKey].data)

                expect(state[stateKey]).toHaveProperty("id", 1);
                expect(state[stateKey]).toHaveProperty("failed", false);
                expect(state[stateKey]).toHaveProperty("inProgress", false);
                expect(state[stateKey]).toHaveProperty("succeeded", true);
                expect(state[stateKey]).toHaveProperty("error", undefined);

                const dataBroker = state[stateKey].data[0];

                if (dataBroker) {
                    expect(dataBroker).toHaveProperty("isWaiting", true);
                    expect(dataBroker).toHaveProperty("isReady", false);
                    expect(dataBroker).toHaveProperty("isFailed", false);
                    expect(dataBroker).toHaveProperty("ui.type.name", "AWS");
                    expect(dataBroker).toHaveProperty("ui.type.displayName", "AWS Data Broker");
                }
            });
        }
    });
}