import {
    tagsToString,
    stringToTags,
    removeKeyFromPath,
    createStringFromEndPointObject
} from "./../../src/utils/helpers";
import {formatActivityValues} from "../../src/utils/helpers";

describe('createStringFromEndPointObject', () => {
    const host = '1.1.1.1';
    const nfsExport = 'export';
    const path = 'folder1/folder2';
    const bucket = 'bucket';
    const port = '80';

    function validateStringCreatedFromEndPoint(endPoint, expectedString) {
        expect(createStringFromEndPointObject(endPoint)).toEqual(expectedString);
    }

    function buildEndPoint(protocol, details) {
        return {protocol, [protocol]: details};
    }

    function buildFileSystemDetailObject(protocol, host, share, path) {
        return (protocol === 'nfs') ? {host, export: share, path} : {host, share, path};
    }

    function buildStorageDetailObject(protocol, bucket, prefix, host, port) {
        return (protocol === 's3') ? {bucket, prefix} : {bucket, prefix, host, port};
    }

    test('nfs endPoint object with no path', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('nfs', buildFileSystemDetailObject('nfs', host, nfsExport)), `nfs://${host}/${nfsExport}`);
    });

    test('nfs endPoint object with path', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('nfs', buildFileSystemDetailObject('nfs', host, nfsExport, path)), `nfs://${host}/${nfsExport}/${path}`);
    });

    test('cifs endPoint object with no path', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('cifs', buildFileSystemDetailObject('cifs', host, nfsExport)), `cifs://${host}/${nfsExport}`);
    });

    test('cifs endPoint object with path', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('cifs', buildFileSystemDetailObject('cifs', host, nfsExport, path)), `cifs://${host}/${nfsExport}/${path}`);
    });

    test('s3 endPoint object with no prefix', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('s3', buildStorageDetailObject('s3', bucket)), `s3://${bucket}`);
    });

    test('s3 endPoint object with prefix', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('s3', buildStorageDetailObject('s3', bucket, path)), `s3://${bucket}/${path}`);
    });

    test('sgws endPoint object with no prefix and no port', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('sgws', buildStorageDetailObject('sgws', bucket, undefined, host)), `sgws://${host}/${bucket}`);
    });

    test('sgws endPoint object with no prefix and port', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('sgws', buildStorageDetailObject('sgws', bucket, undefined, host, port)), `sgws://${host}:${port}/${bucket}`);
    });
    test('sgws endPoint object with  prefix and port', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('sgws', buildStorageDetailObject('sgws', bucket, path, host, port)), `sgws://${host}:${port}/${bucket}/${path}`);
    });

});

describe("tagsToString", () => {
    test("should return the right string for a tags array with one tag", () => {
        expect(tagsToString([{key: "key", value: "value"}])).toEqual("key=value");
    });

    test("should return the right string for a tags array with more than one tag", () => {
        expect(tagsToString([{key: "key", value: "value"}, {
            key: "key2",
            value: "value2"
        }])).toEqual("key=value,key2=value2");
    });

    test("should return 'no tags' when there are not tags", () => {
        expect(tagsToString([{key: undefined, value: undefined}])).toEqual("noTags");
    });

    test("should skip empty tags and not include them in the list", () => {
        expect(tagsToString([{key: "key", value: "value"}, {key: undefined, value: undefined}, {
            key: "key2",
            value: "value2"
        }])).toEqual("key=value,key2=value2");
    });
    test("should support empty array", () => {
        expect(tagsToString([])).toEqual("noTags");
    });
    test("should support array with empty objects", () => {
        expect(tagsToString([{}, {}])).toEqual("noTags");
    });
    test("should support array with null objects", () => {
        expect(tagsToString([null, null])).toEqual("noTags");
    });
});

describe("stringToTags", () => {
    test("should return the right array with one object for one tagString", () => {
        expect(stringToTags("key=value")).toEqual([{key: "key", value: "value"}]);
    });

    test("should return the right array with more than one tag for string with more than one tag", () => {
        expect(stringToTags("key=value,key2=value2")).toEqual([{key: "key", value: "value"}, {
            key: "key2",
            value: "value2"
        }]);
    });
});

describe("removeKeyFromPath", () => {
    test("should return the right path - /aa/bb/cc - key in the middle of path", () => {
        expect(removeKeyFromPath("/aa/bb/cc", "/bb")).toEqual("/aa/bb/cc");
    });
    test("should return the right path - /aa/bb/cc - path ends in key", () => {
        expect(removeKeyFromPath("/aa/bb/cc", "/cc")).toEqual("/aa/bb/cc");
    });
    test("should return the right path - /bb/cc - path starts in key", () => {
        expect(removeKeyFromPath("/aa/bb/cc", "/aa")).toEqual("/bb/cc");
    });
    test("should return the right path - /aa/bb/cc- - key is twice in path", () => {
        expect(removeKeyFromPath("/aa/aa/bb/cc", "/aa")).toEqual("/aa/bb/cc");
    });
    test("should return the right path - /bb/cc/aa- key is twice in path", () => {
        expect(removeKeyFromPath("/aa/bb/cc/aa", "/aa")).toEqual("/bb/cc/aa");
    });
    test("should return the right path - key not in path", () => {
        expect(removeKeyFromPath("/bb/cc/aa", "guy")).toEqual("/bb/cc/aa");
    });


});

describe("formatActivityValues", () => {
    const basicActivity = {
        bytesCopied: 5581252300000,
        bytesFailed: 253000000,
        bytesMarkedForCopy: 11163010600000,
        bytesMarkedForGrace: 0,
        bytesMarkedForRemove: 0,
        bytesRemoved: 890,
        bytesRemovedFailed: 0,
        bytesScanned: 5581505300445,
        dirsCopied: 4,
        dirsFailed: 0,
        dirsFailedToScan: 6,
        dirsMarkedForCopy: 4,
        dirsMarkedForGrace: 0,
        dirsMarkedForRemove: 0,
        dirsRemoved: 0,
        dirsScanned: 2,
        filesCopied: 692,
        filesFailed: 4,
        filesMarkedForCopy: 1392,
        filesMarkedForGrace: 0,
        filesMarkedForIgnore: 3,
        filesMarkedforRemove: 0,
        filesRemoved: 6,
        filesRemovedFailed: 0,
        filesScanned: 699
    };
    test("should format the activity values correctly", () => {
        expect(formatActivityValues(basicActivity)).toEqual({
            "formattedBytesCopied": {"size": "5.08", "suffix": "TB"},
            "formattedBytesFailed": {"size": "241.28", "suffix": "MB"},
            "formattedBytesMarkedForCopy": {"size": "10.15", "suffix": "TB"},
            "formattedBytesMarkedForRemove": {"size": "0", "suffix": "B"},
            "formattedBytesRemoved": {"size": "890", "suffix": "B"},
            "formattedBytesScanned": {"size": "5.08", "suffix": "TB"},
            "formattedDirsFailedToScan": "6",
            "formattedDirsMarkedForCopy": "4",
            "formattedDirsMarkedForRemove": "0",
            "formattedDirsScanned": "2",
            "formattedFilesCopied": "692",
            "formattedFilesFailed": "4",
            "formattedFilesMarkedForCopy": "1.4k",
            "formattedFilesMarkedForRemove": "0",
            "formattedFilesRemoved": "6",
            "formattedFilesScanned": "699"
        });
    });
    test("should add the bytesRemovedFailed and FilesRemovedFailed to the failed counters", () => {
        const activityWithFailedRemoved = Object.assign({}, basicActivity, {
            filesRemovedFailed: 10,
            bytesRemovedFailed: 23454524234
        });
        expect(formatActivityValues(activityWithFailedRemoved)).toEqual({
            "formattedBytesCopied": {"size": "5.08","suffix": "TB"},
            "formattedBytesFailed": {"size": "22.08", "suffix": "GB"},
            "formattedBytesMarkedForCopy": {"size": "10.15", "suffix": "TB"},
            "formattedBytesMarkedForRemove": {"size": "0", "suffix": "B"},
            "formattedBytesRemoved": {"size": "890", "suffix": "B"},
            "formattedBytesScanned": {"size": "5.08", "suffix": "TB"},
            "formattedDirsFailedToScan": "6",
            "formattedDirsMarkedForCopy": "4",
            "formattedDirsMarkedForRemove": "0",
            "formattedDirsScanned": "2",
            "formattedFilesCopied": "692",
            "formattedFilesFailed": "14",
            "formattedFilesMarkedForCopy": "1.4k",
            "formattedFilesMarkedForRemove": "0",
            "formattedFilesRemoved": "6",
            "formattedFilesScanned": "699"
        });
    });

});